# ===================================================
# ===================================================
# Code to reproduce Figure 3 and Table 5 in application
# ===================================================
# ===================================================



# ===================================================
# Load relevant data
# ===================================================

library(xts)    # for coredata()-command


# Get the files names
files = list.files(path = "./Data", pattern = "*.csv")   
# First apply read.csv, then rbind
myfiles = do.call(cbind, lapply(files, function(x) read.csv(paste0("./Data/",x), stringsAsFactors = FALSE)))



# 1.1 AAPL returns
Datum <- myfiles[, 1]                     # read in dates ...
Datum <- as.Date(Datum)                   # ... and convert to date format
price <- myfiles[, 6]                     # read in (adjusted) price history
price <- as.numeric(as.character(price))  # convert "factor" to "numeric"
price <- xts(x = price, order.by = Datum) # create xts-object for easier manipulation
price <- price[!is.na(price)]             # delete NA's
lr    <- -diff(log(price))[-1]             # compute log-losses with adjusted closing prices

# 1.2 All other returns
for(i in 2 : 5){
  Datum <- myfiles[, 1 + (i-1) * 7]         # read in dates ...
  Datum <- as.Date(Datum)                   # ... and convert to date format
  price <- myfiles[, 6 + (i-1) * 7]         # read in (adjusted) price history
  price <- as.numeric(as.character(price))  # convert "factor" to "numeric"
  price <- xts(x = price, order.by = Datum) # create xts-object for easier manipulation
  price <- price[!is.na(price)]             # delete NA's
  lr    <- merge(lr, -diff(log(price))[-1], all = FALSE)  # compute log-losses with adjusted closing prices and merge
}

Datum <- index(lr)   # save dates for later use
V     <- t(coredata(lr))
N     <- length(Datum)



# ===================================================
# Carry out computations
# ===================================================

library(snowfall)     # Parallel computing

sfInit(parallel = TRUE, cpus = 3, type = "SOCK")
#sfInit(parallel = FALSE)   # useful for trouble-shooting
sfLibrary(fGarch)
source("RfnsAppl.R")     # contains functions "VaR.comparison", "dm95.test", "loss.VaR", "VaR.forecast"
sfExport("VaR.comparison", "V", "dm95.test", "loss.VaR", "VaR.forecast")


wrapper <- function(c, n, h=1){
   VaR.comparison(c, n, h)
}


start <- Sys.time()
# Evaluation of VaR forecasts with quantile score
res.500  <- sfLapply(1:5, wrapper, n = 500)
res.1000 <- sfLapply(1:5, wrapper, n = 1000)
res.2000 <- sfLapply(1:5, wrapper, n = 2000)

# Evaluation of VaR forecasts with alternative score (results not reported in paper)
res.500.h0  <- sfLapply(1:5, wrapper, n = 500, h=0)
res.1000.h0 <- sfLapply(1:5, wrapper, n = 1000, h=0)
res.2000.h0 <- sfLapply(1:5, wrapper, n = 2000, h=0)

print(Sys.time() - start)


sfStop()


# ===================================================
# Table 5: Score ratios of VaR forecasts
# ===================================================

QCRPS.NP.500  <- QCRPS.NP.1000  <- QCRPS.NP.2000  <- rep.int(NA, 5)
MAX.NP.500    <- MAX.NP.1000    <- MAX.NP.2000    <- rep.int(NA, 5)
QCRPS.MAX.500 <- QCRPS.MAX.1000 <- QCRPS.MAX.2000 <- rep.int(NA, 5)

for(i in 1:5){
   # n = 500
   cat("i = ", i, "\n")
   print("n = 500")
   print("res.500")
   print(res.500[[i]]$Results)
   
   print("res.500.h0")
   print(res.500.h0[[i]]$Results)
   
   # n = 1000
   print("n = 1000")
   print(res.1000[[i]]$Results)
   
   print("res.1000.h0")
   print(res.1000.h0[[i]]$Results)
   
   # n = 2000
   print("n = 2000")
   print(res.2000[[i]]$Results)
   
   print("res.2000.h0")
   print(res.2000.h0[[i]]$Results)
}


# ===================================================
# Figure 3: Rolling window estimates and VaR forecasts
# ===================================================

n <-  2000
Datum.sub      <- Datum[(n+1) : N]
GARCH.param    <- res.2000[[1]]$GARCH.param
CVaR.forecasts <- res.2000[[1]]$CVaR.forecasts

par(oma=c(4,1,2,1), mar=c(0,5,0,1), mfrow=c(5,1), pch=16)

plot(Datum.sub, GARCH.param[1, ]*10^3, type="l", ylab=expression(hat(ring(omega)) * "*10"^"3"), xlab="", xaxt = "n", las=1)
plot(Datum.sub, GARCH.param[2, ], type="l", ylab=expression(hat(ring(alpha))), xlab="", xaxt = "n", las=1, ylim=c(0, 0.33))
plot(Datum.sub, GARCH.param[3, ], type="l", ylab=expression(hat(ring(beta))),  xlab="", xaxt = "n", las=1, ylim=c(0, 1.03))
plot(Datum.sub, GARCH.param[4, ], type="l", ylab=expression(hat(alpha)),       xlab="", xaxt = "n", las=1)
plot(Datum.sub, V[1, (n+1) : N],  type="l", ylab="log-losses",                 xlab="",             las=1)
lines(Datum.sub, CVaR.forecasts,  type="l", lty="dotted")

mtext(text="Date", side=1, line=3, outer=TRUE)



###################################
##### 2. Conditional Analysis #####
###################################

VaR.comparison <- function(c, n, h){    # h=1: 1-homogenous scoring function (standard), h=0: 0-homogenous scoring function
    # 1. Prepare data
   Results <- matrix(NA, nrow=2, ncol=12, 
                     dimnames = list(c("MAX", "NP"), 
                                     c("5% Quantile Score Ratio", "DM", "1% QSR", "DM", "0.5% QSR", "DM", "0.1% QSR", "DM", 
                                       "0.05% QSR", "DM", "0.001% QSR", "DM"))) 
    cat("c = ", c, "\n")
    lr.DAX    <- V[c, ]                 # Investigate log-returns
    N         <- length(lr.DAX)
    
    
    # 2. Forecast CVaR and CES
    
    alpha    <- c(0.05, 0.01, 0.005, 0.001, 0.0005, 0.00001)
    no.alpha <- length(alpha)
    
    CVaR.forecasts.QCRPS <- matrix(NA, nrow = no.alpha, ncol = (N-n))
    CVaR.forecasts.MAX   <- matrix(NA, nrow = no.alpha, ncol = (N-n)) 
    CVaR.forecasts.NP    <- matrix(NA, nrow = no.alpha, ncol = (N-n)) 
    
    k.QCRPS <- rep.int(NA, times=(N-n))
    k.MAX   <- rep.int(NA, times=(N-n)) 
    
    GARCH.param <- matrix(NA, nrow=4, ncol= (N-n), dimnames = list(c("omega.hat", "alpha.hat", "beta.hat", "tail index.hat"), 1:(N-n))) 
    
    
    for(j in n : (N-1) ){
      forecast            <- VaR.forecast(lr.DAX[(j-n+1) : j], alpha, nu = 10)
      
      k.QCRPS[(j-n+1)] <- forecast$k.QCRPS
      k.MAX[(j-n+1)]   <- forecast$k.MAX
      
      GARCH.param[1:3, (j-n+1)] <- forecast$theta.hat        # save GARCH(1,1) parameter estimates
      GARCH.param[4, (j-n+1)]   <- forecast$alpha.hat        # save tail indices implied by GARCH(1,1) parameter estimates
      
      for(a in 1 : no.alpha){
        CVaR.forecasts.QCRPS[a, (j-n+1)] <- forecast$VaR.QCRPS[a]
        CVaR.forecasts.MAX[a, (j-n+1)]   <- forecast$VaR.MAX[a]
        CVaR.forecasts.NP[a, (j-n+1)]    <- forecast$VaR.NP[a] 
      }
    }
    k <- matrix(c(mean(k.MAX), mean(k.QCRPS)), nrow=1, ncol=2, dimnames=list(c(""), c("k.Q", "k.QCRPS")) )
    
    # 3. Comparative Backtest
    
    X <- lr.DAX[(n+1):N]
    
    for(a in 1:no.alpha){
       
        # 5. Diebold Mariano (1995) Test of equal predictive ability
        
        l1.VaR <- loss.VaR(data = X, VaR = CVaR.forecasts.QCRPS[a, ], 1-alpha[a], h)
        l2.VaR <- loss.VaR(data = X, VaR = CVaR.forecasts.MAX[a, ],   1-alpha[a], h)
        l3.VaR <- loss.VaR(data = X, VaR = CVaR.forecasts.NP[a, ],    1-alpha[a], h)
        
        Results[1, 1 + (a-1)*2] <- sum(l1.VaR) / sum(l2.VaR)
        Results[2, 1 + (a-1)*2] <- sum(l1.VaR) / sum(l3.VaR)
        
        # print("VaR:")
        dm.l1l2 <- dm95.test(l1.VaR, l2.VaR, "two.sided")
        dm.l1l3 <- dm95.test(l1.VaR, l3.VaR, "two.sided")
        
        Results[1, 2 + (a-1)*2] <- dm.l1l2
        Results[2, 2 + (a-1)*2] <- dm.l1l3
    }
    
    # 6. Summarize Results in Matrix
    return( list(Results = Results, GARCH.param = GARCH.param, CVaR.forecasts = CVaR.forecasts.MAX[2, ], k=k) )
}



##############################################################
############### Functions to replicate Table 3 ############### 
##############################################################


# Function to generate conditional VaR forecasts

VaR.forecast <- function(X, alpha, nu){  # VaR forecasts using Chan et al. (2007) and Hoga (2017+)
  n <- length(X) - nu
  no.alpha <- length(alpha)
  
  #####################################
  
  # 1. Fit model to generate conditional risk forecasts
  # 1.1. Estimate GARCH(1,1) model using standard QMLE for log-returns
  outp      <- garchFit(formula = ~garch(1,1), data = X, init.rec = "mci", cond.dist = "QMLE",
                        include.mean = FALSE, algorithm = "lbfgsb", trace = FALSE)
  param     <- outp@fit$matcoef
  theta.hat <- as.numeric(param[ ,1])


  # 1.2. Compute residuals U.hat
  X         <- X[(nu+1) : (nu+n)]
  sigma.hat <- outp@sigma.t[(nu+1) : (nu+n)]  # discard first nu volatility estimates, due to initialization effects
  U.hat     <- X / sigma.hat
  sort.U    <- sort.int(U.hat, decreasing = TRUE, method = "quick")

  # 1.3 Volatility estimate
  sigma.hat.n1 <- sqrt( theta.hat[1] + theta.hat[2] * X[n]^2 + theta.hat[3] * sigma.hat[n]^2 )
  
  
  # 2. Compute optimal k* using [Dea16+, p. 16] (Tail Index Estimation: Quantile Driven Threshold Selection)
  
  k.max <- floor(n^0.6)
  
  sup.MAX <- rep.int(NA, k.max)
  QCRPS   <- H <- rep.int(0, k.max)
  for(k in 1 : k.max){
    H[k]      <- 1/k * sum(log( sort.U[1 : (k+1)] / sort.U[(k+1)] ))  # Hill estimate
    Q.k       <- (k/(1 : k.max))^( H[k] ) * sort.U[(k+1)]             # Weissman estimate
    deviation <- sort.U[2 : (k.max+1)] - Q.k                          # deviation between empirical and Weissman quantile estimates
    
    sup.MAX[k] <- max( abs(deviation) )                               # Choice of [Dea16+]

    for(i in 1 : k.max){
      QCRPS[k] <- QCRPS[k] + sum( (1*(sort.U <= Q.k[i]) - (1 - i/n)) * (Q.k[i] - sort.U) )
    }
  }
   # 3. Compute Risk Measure Forecasts
  k.MAX   <- which.min( sup.MAX )
  k.QCRPS <- which.min( QCRPS )
   
  H.MAX   <- sort.U[k.MAX+1]   * ((n*alpha) / k.MAX)^(-H[k.MAX])     # H EQ estimator
  H.QCRPS <- sort.U[k.QCRPS+1] * ((n*alpha) / k.QCRPS)^(-H[k.QCRPS]) # H EQ estimator
  H.NP    <- rep.int(NA, no.alpha)                  
  for(a in 1:no.alpha){
    H.NP[a]       <- sort.U[floor(n*alpha[a])+1]
  }
  
  CVaR.MAX   <- sigma.hat.n1 * H.MAX     
  CVaR.QCRPS <- sigma.hat.n1 * H.QCRPS   
  CVaR.NP    <- sigma.hat.n1 * H.NP      
  
  
  return( list( VaR.MAX = c(CVaR.MAX), VaR.QCRPS = c(CVaR.QCRPS), VaR.NP = c(CVaR.NP),
                k.MAX = c(k.MAX), k.QCRPS = c(k.QCRPS),
                theta.hat = c(theta.hat), alpha.hat = 1 / H[k.MAX]) )
}


#######################

# VaR loss via tick loss function

loss.VaR <- function(data, VaR, alpha, h){  # h=1 corresponds to 1-homogenous choice, h=0 to 0-homogenous
   S <- numeric(length(VaR))
   if(h==1){
      S <- (data - VaR) * (alpha - 1*(data <= VaR)) 
   }
   # ind1 = (data>VaR) & (VaR>0)
   # ind2 = (data<VaR) & (VaR>0)
   if(h==0){
      S <- ( exp(data) / (1+exp(data)) - exp(VaR) / (1+exp(VaR)) ) * (alpha - 1*(data <= VaR))             
   }
  return(S)
}


#######################

# Diebold-Mariano (1995) test of equal predicitve ability

dm95.test <- function (loss1, loss2, alternative = c("two.sided", "less", "greater"), h = 1){
  alternative <- match.arg(alternative)
  d <- c(loss1) - c(loss2)
  d.cov <- acf(d, na.action = na.omit, lag.max = h - 1, type = "covariance",
               plot = FALSE)$acf[, , 1]
  d.var <- sum(c(d.cov[1], 2 * d.cov[-1]))/length(d)
  dv <- d.var
  if (dv > 0)
    STATISTIC <- mean(d, na.rm = TRUE)/sqrt(dv)
  else if (h == 1)
    stop("Variance of DM statistic is zero")
  else {
    warning("Variance is negative, using horizon h=1")
    return(dm95.test(loss1, loss2, alternative, h = 1))
  }
  n <- length(d)
  k <- ((n + 1 - 2 * h + (h/n) * (h - 1))/n)^(1/2)
  STATISTIC <- STATISTIC * k
  names(STATISTIC) <- "DM"
  if (alternative == "two.sided")
    PVAL <- 2 * pt(-abs(STATISTIC), df = n - 1)
  else if (alternative == "less")
    PVAL <- pt(STATISTIC, df = n - 1)
  else if (alternative == "greater")
    PVAL <- pt(STATISTIC, df = n - 1, lower.tail = FALSE)
  return(PVAL)
}

dm95.testown <- function(loss1, loss2){
   num <- mean(loss1-loss2)
   denom <- sqrt(var(loss1-loss2) / length(loss1))
   ts <- num/denom
   return(2*(1-pnorm(abs(ts))))
}

l1 <- rnorm(500)
l2 <- rnorm(500)

dm95.testown(l1, l2)
dm95.test(l1, l2, "two.sided")

################################################################################


