# ===================================================
# ===================================================
# Code to reproduce data and figures in simulations
# ===================================================
# ===================================================




library(snowfall)     # Parallel computing

sfInit(parallel = TRUE, cpus = 30, type = "SOCK")
#sfInit(parallel = FALSE)   # useful for trouble-shooting
sfLibrary(actuar)
sfLibrary(evd)
sfLibrary(fGarch)
sfLibrary(stabledist)
source("RfnsSim.R")    # file contains the functions "EVI.k" and "RMSE"
sfExport("EVI.k", "RMSE")


# ===========================================================
# Wrapper function for parallelization
# ===========================================================

# Inputs: 
# d: specifies the DGP
# n: sample size
# estimator: either "Hill" for Hill (1975) estimator or "GI" for Gabaix & Ibragimov (2011) estimator

wrapper <- function(d, n, estimator){
  RMSE(d, n, reps=10000, k.max=floor(n^0.6), estimator)    # takes 12 hours on 30 cores for Hill estimator
}


start <- Sys.time()

# Results for Hill (1975) estimator reported in main paper
res.n.500  <- sfLapply(c(1:30), wrapper, n = 500,  estimator="Hill")
res.n.1000 <- sfLapply(c(1:30), wrapper, n = 1000, estimator="Hill")
res.n.2000 <- sfLapply(c(1:30), wrapper, n = 2000, estimator="Hill")
res.n.5000 <- sfLapply(c(1:30), wrapper, n = 5000, estimator="Hill")

# Optional: Results for Gabaix & Ibragimov (2011) estimator reported in "Online Supplemental Appendices"
# res.n.500  <- sfLapply(1:30, wrapper, n = 500,  estimator="GI")
# res.n.1000 <- sfLapply(1:30, wrapper, n = 1000, estimator="GI")
# res.n.2000 <- sfLapply(1:30, wrapper, n = 2000, estimator="GI")
# res.n.5000 <- sfLapply(1:30, wrapper, n = 5000, estimator="GI")
print(Sys.time() - start)

sfStop()

# Save all results
RMSE.min <- Bias.min <- k.min <- SD.k.min <- matrix( NA, nrow=31, ncol=8, dimnames= list(c(1:30, "SUM"), c("Dea16 500", "SCR", "Dea16 1000", "SCR", "Dea16 2000", "SCR", "Dea16 5000", "SCR")))
MSE.99.99 <- MSE.99.95 <- MSE.99.9 <- MSE.99.5 <- MSE.99 <- MSE.975 <- MSE.95 <- MSE.90 <- matrix( NA, nrow=31, ncol=8, dimnames= list(c(1:30, "SUM"), c("Dea16 500", "SCR", "Dea16 1000", "SCR", "Dea16 2000", "SCR", "Dea16 5000", "SCR")))
MSE.ES.99.99 <- MSE.ES.99.95 <- MSE.ES.99.9 <- MSE.ES.99.5 <- MSE.ES.99 <- MSE.ES.975 <- MSE.ES.95 <- MSE.ES.90 <- matrix( NA, nrow=31, ncol=8, dimnames= list(c(1:30, "SUM"), c("Dea16 500", "SCR", "Dea16 1000", "SCR", "Dea16 2000", "SCR", "Dea16 5000", "SCR")))
  
for(i in 1:30){
  k.min[i, 1:2] <-  res.n.500[[i]]$k.sim
  k.min[i, 3:4] <- res.n.1000[[i]]$k.sim
  k.min[i, 5:6] <- res.n.2000[[i]]$k.sim
  k.min[i, 7:8] <- res.n.5000[[i]]$k.sim
  
  SD.k.min[i, 1:2] <-  res.n.500[[i]]$SD.k.sim
  SD.k.min[i, 3:4] <- res.n.1000[[i]]$SD.k.sim
  SD.k.min[i, 5:6] <- res.n.2000[[i]]$SD.k.sim
  SD.k.min[i, 7:8] <- res.n.5000[[i]]$SD.k.sim
  
  # gamma estimates
  RMSE.min[i, 1:2] <-  res.n.500[[i]]$RMSE.sim
  RMSE.min[i, 3:4] <- res.n.1000[[i]]$RMSE.sim
  RMSE.min[i, 5:6] <- res.n.2000[[i]]$RMSE.sim
  RMSE.min[i, 7:8] <- res.n.5000[[i]]$RMSE.sim
  
  Bias.min[i, 1:2] <-  res.n.500[[i]]$Bias.sim
  Bias.min[i, 3:4] <- res.n.1000[[i]]$Bias.sim
  Bias.min[i, 5:6] <- res.n.2000[[i]]$Bias.sim
  Bias.min[i, 7:8] <- res.n.5000[[i]]$Bias.sim
  
  # VaR estimates
  MSE.99.99[i, 1:2] <-  res.n.500[[i]]$MSE.VaR.sim[8, ]
  MSE.99.99[i, 3:4] <- res.n.1000[[i]]$MSE.VaR.sim[8, ]
  MSE.99.99[i, 5:6] <- res.n.2000[[i]]$MSE.VaR.sim[8, ]
  MSE.99.99[i, 7:8] <- res.n.5000[[i]]$MSE.VaR.sim[8, ]
  
  MSE.99.95[i, 1:2] <-  res.n.500[[i]]$MSE.VaR.sim[7, ]
  MSE.99.95[i, 3:4] <- res.n.1000[[i]]$MSE.VaR.sim[7, ]
  MSE.99.95[i, 5:6] <- res.n.2000[[i]]$MSE.VaR.sim[7, ]
  MSE.99.95[i, 7:8] <- res.n.5000[[i]]$MSE.VaR.sim[7, ]
  
  MSE.99.9[i, 1:2] <-  res.n.500[[i]]$MSE.VaR.sim[6, ]
  MSE.99.9[i, 3:4] <- res.n.1000[[i]]$MSE.VaR.sim[6, ]
  MSE.99.9[i, 5:6] <- res.n.2000[[i]]$MSE.VaR.sim[6, ]
  MSE.99.9[i, 7:8] <- res.n.5000[[i]]$MSE.VaR.sim[6, ]
  
  MSE.99.5[i, 1:2] <-  res.n.500[[i]]$MSE.VaR.sim[5, ]
  MSE.99.5[i, 3:4] <- res.n.1000[[i]]$MSE.VaR.sim[5, ]
  MSE.99.5[i, 5:6] <- res.n.2000[[i]]$MSE.VaR.sim[5, ]
  MSE.99.5[i, 7:8] <- res.n.5000[[i]]$MSE.VaR.sim[5, ]
  
  MSE.99[i, 1:2] <-  res.n.500[[i]]$MSE.VaR.sim[4, ]
  MSE.99[i, 3:4] <- res.n.1000[[i]]$MSE.VaR.sim[4, ]
  MSE.99[i, 5:6] <- res.n.2000[[i]]$MSE.VaR.sim[4, ]
  MSE.99[i, 7:8] <- res.n.5000[[i]]$MSE.VaR.sim[4, ]
  
  MSE.975[i, 1:2] <-  res.n.500[[i]]$MSE.VaR.sim[3, ]
  MSE.975[i, 3:4] <- res.n.1000[[i]]$MSE.VaR.sim[3, ]
  MSE.975[i, 5:6] <- res.n.2000[[i]]$MSE.VaR.sim[3, ]
  MSE.975[i, 7:8] <- res.n.5000[[i]]$MSE.VaR.sim[3, ]
  
  MSE.95[i, 1:2] <-  res.n.500[[i]]$MSE.VaR.sim[2, ]
  MSE.95[i, 3:4] <- res.n.1000[[i]]$MSE.VaR.sim[2, ]
  MSE.95[i, 5:6] <- res.n.2000[[i]]$MSE.VaR.sim[2, ]
  MSE.95[i, 7:8] <- res.n.5000[[i]]$MSE.VaR.sim[2, ]
  
  MSE.90[i, 1:2] <-  res.n.500[[i]]$MSE.VaR.sim[1, ]
  MSE.90[i, 3:4] <- res.n.1000[[i]]$MSE.VaR.sim[1, ]
  MSE.90[i, 5:6] <- res.n.2000[[i]]$MSE.VaR.sim[1, ]
  MSE.90[i, 7:8] <- res.n.5000[[i]]$MSE.VaR.sim[1, ]
  
  # ES estimates
  MSE.ES.99.99[i, 1:2] <-  res.n.500[[i]]$MSE.ES.sim[8, ]
  MSE.ES.99.99[i, 3:4] <- res.n.1000[[i]]$MSE.ES.sim[8, ]
  MSE.ES.99.99[i, 5:6] <- res.n.2000[[i]]$MSE.ES.sim[8, ]
  MSE.ES.99.99[i, 7:8] <- res.n.5000[[i]]$MSE.ES.sim[8, ]
  
  MSE.ES.99.95[i, 1:2] <-  res.n.500[[i]]$MSE.ES.sim[7, ]
  MSE.ES.99.95[i, 3:4] <- res.n.1000[[i]]$MSE.ES.sim[7, ]
  MSE.ES.99.95[i, 5:6] <- res.n.2000[[i]]$MSE.ES.sim[7, ]
  MSE.ES.99.95[i, 7:8] <- res.n.5000[[i]]$MSE.ES.sim[7, ]
  
  MSE.ES.99.9[i, 1:2] <-  res.n.500[[i]]$MSE.ES.sim[6, ]
  MSE.ES.99.9[i, 3:4] <- res.n.1000[[i]]$MSE.ES.sim[6, ]
  MSE.ES.99.9[i, 5:6] <- res.n.2000[[i]]$MSE.ES.sim[6, ]
  MSE.ES.99.9[i, 7:8] <- res.n.5000[[i]]$MSE.ES.sim[6, ]

  MSE.ES.99.5[i, 1:2] <-  res.n.500[[i]]$MSE.ES.sim[5, ]
  MSE.ES.99.5[i, 3:4] <- res.n.1000[[i]]$MSE.ES.sim[5, ]
  MSE.ES.99.5[i, 5:6] <- res.n.2000[[i]]$MSE.ES.sim[5, ]
  MSE.ES.99.5[i, 7:8] <- res.n.5000[[i]]$MSE.ES.sim[5, ]

  MSE.ES.99[i, 1:2] <-  res.n.500[[i]]$MSE.ES.sim[4, ]
  MSE.ES.99[i, 3:4] <- res.n.1000[[i]]$MSE.ES.sim[4, ]
  MSE.ES.99[i, 5:6] <- res.n.2000[[i]]$MSE.ES.sim[4, ]
  MSE.ES.99[i, 7:8] <- res.n.5000[[i]]$MSE.ES.sim[4, ]

  MSE.ES.975[i, 1:2] <-  res.n.500[[i]]$MSE.ES.sim[3, ]
  MSE.ES.975[i, 3:4] <- res.n.1000[[i]]$MSE.ES.sim[3, ]
  MSE.ES.975[i, 5:6] <- res.n.2000[[i]]$MSE.ES.sim[3, ]
  MSE.ES.975[i, 7:8] <- res.n.5000[[i]]$MSE.ES.sim[3, ]

  MSE.ES.95[i, 1:2] <-  res.n.500[[i]]$MSE.ES.sim[2, ]
  MSE.ES.95[i, 3:4] <- res.n.1000[[i]]$MSE.ES.sim[2, ]
  MSE.ES.95[i, 5:6] <- res.n.2000[[i]]$MSE.ES.sim[2, ]
  MSE.ES.95[i, 7:8] <- res.n.5000[[i]]$MSE.ES.sim[2, ]

  MSE.ES.90[i, 1:2] <-  res.n.500[[i]]$MSE.ES.sim[1, ]
  MSE.ES.90[i, 3:4] <- res.n.1000[[i]]$MSE.ES.sim[1, ]
  MSE.ES.90[i, 5:6] <- res.n.2000[[i]]$MSE.ES.sim[1, ]
  MSE.ES.90[i, 7:8] <- res.n.5000[[i]]$MSE.ES.sim[1, ]
}
k.min[31, ]     <- colMeans(k.min[1:30, ])
SD.k.min[31, ]  <- colMeans(SD.k.min[1:30, ])
RMSE.min[31, ]  <- colSums(RMSE.min[1:30, ])
Bias.min[31, ]  <- colSums(abs(Bias.min[1:30, ]))
MSE.99.99[31, ] <- colSums(MSE.99.99[1:30, ])
MSE.99.95[31, ] <- colSums(MSE.99.95[1:30, ])
MSE.99.9[31, ]  <- colSums(MSE.99.9[1:30, ])
MSE.99.5[31, ]  <- colSums(MSE.99.5[1:30, ])
MSE.99[31, ]    <- colSums(MSE.99[1:30, ])
MSE.95[31, ]    <- colSums(MSE.95[1:30, ])
MSE.90[31, ]    <- colSums(MSE.90[1:30, ])
MSE.ES.99.99[31, ] <- colSums(MSE.ES.99.99[1:30, ])
MSE.ES.99.95[31, ] <- colSums(MSE.ES.99.95[1:30, ])
MSE.ES.99.9[31, ]  <- colSums(MSE.ES.99.9[1:30, ])
MSE.ES.99.5[31, ]  <- colSums(MSE.ES.99.5[1:30, ])
MSE.ES.99[31, ]    <- colSums(MSE.ES.99[1:30, ])
MSE.ES.95[31, ]    <- colSums(MSE.ES.95[1:30, ])
MSE.ES.90[31, ]    <- colSums(MSE.ES.90[1:30, ])



# ===================================================
# Figure 1: RMSE ratios for VaR estimation for (M1), (M2) and (M6)
# ===================================================

############################
# t_3 and t_5 distribution #
############################

RMSE.n.500.t3 <- RMSE.n.500.t5 <- RMSE.n.2000.t3 <- RMSE.n.2000.t5 <- RMSE.n.5000.t3 <- RMSE.n.5000.t5 <- matrix(NA, nrow=8, ncol=2)

RMSE.n.500.t3[1, ]  <- MSE.90[2, 1:2]
RMSE.n.2000.t3[1, ] <- MSE.90[2, 5:6]
RMSE.n.5000.t3[1, ] <- MSE.90[2, 7:8]
RMSE.n.500.t3[2, ]  <- MSE.95[2, 1:2]
RMSE.n.2000.t3[2, ] <- MSE.95[2, 5:6]
RMSE.n.5000.t3[2, ] <- MSE.95[2, 7:8]
RMSE.n.500.t3[3, ]  <- MSE.975[2, 1:2]
RMSE.n.2000.t3[3, ] <- MSE.975[2, 5:6]
RMSE.n.5000.t3[3, ] <- MSE.975[2, 7:8]
RMSE.n.500.t3[4, ]  <- MSE.99[2, 1:2]
RMSE.n.2000.t3[4, ] <- MSE.99[2, 5:6]
RMSE.n.5000.t3[4, ] <- MSE.99[2, 7:8]
RMSE.n.500.t3[5, ]  <- MSE.99.5[2, 1:2]
RMSE.n.2000.t3[5, ] <- MSE.99.5[2, 5:6]
RMSE.n.5000.t3[5, ] <- MSE.99.5[2, 7:8]
RMSE.n.500.t3[6, ]  <- MSE.99.9[2, 1:2]
RMSE.n.2000.t3[6, ] <- MSE.99.9[2, 5:6]
RMSE.n.5000.t3[6, ] <- MSE.99.9[2, 7:8]
RMSE.n.500.t3[7, ]  <- MSE.99.95[2, 1:2]
RMSE.n.2000.t3[7, ] <- MSE.99.95[2, 5:6]
RMSE.n.5000.t3[7, ] <- MSE.99.95[2, 7:8]
RMSE.n.500.t3[8, ]  <- MSE.99.99[2, 1:2]
RMSE.n.2000.t3[8, ] <- MSE.99.99[2, 5:6]
RMSE.n.5000.t3[8, ] <- MSE.99.99[2, 7:8]


RMSE.n.500.t5[1, ]  <- MSE.90[3, 1:2]
RMSE.n.2000.t5[1, ] <- MSE.90[3, 5:6]
RMSE.n.5000.t5[1, ] <- MSE.90[3, 7:8]

RMSE.n.500.t5[2, ]  <- MSE.95[3, 1:2]
RMSE.n.2000.t5[2, ] <- MSE.95[3, 5:6]
RMSE.n.5000.t5[2, ] <- MSE.95[3, 7:8]

RMSE.n.500.t5[3, ]  <- MSE.975[3, 1:2]
RMSE.n.2000.t5[3, ] <- MSE.975[3, 5:6]
RMSE.n.5000.t5[3, ] <- MSE.975[3, 7:8]

RMSE.n.500.t5[4, ]  <- MSE.99[3, 1:2]
RMSE.n.2000.t5[4, ] <- MSE.99[3, 5:6]
RMSE.n.5000.t5[4, ] <- MSE.99[3, 7:8]

RMSE.n.500.t5[5, ]  <- MSE.99.5[3, 1:2]
RMSE.n.2000.t5[5, ] <- MSE.99.5[3, 5:6]
RMSE.n.5000.t5[5, ] <- MSE.99.5[3, 7:8]

RMSE.n.500.t5[6, ]  <- MSE.99.9[3, 1:2]
RMSE.n.2000.t5[6, ] <- MSE.99.9[3, 5:6]
RMSE.n.5000.t5[6, ] <- MSE.99.9[3, 7:8]

RMSE.n.500.t5[7, ]  <- MSE.99.95[3, 1:2]
RMSE.n.2000.t5[7, ] <- MSE.99.95[3, 5:6]
RMSE.n.5000.t5[7, ] <- MSE.99.95[3, 7:8]

RMSE.n.500.t5[8, ]  <- MSE.99.99[3, 1:2]
RMSE.n.2000.t5[8, ] <- MSE.99.99[3, 5:6]
RMSE.n.5000.t5[8, ] <- MSE.99.99[3, 7:8]

##########################################
# Frechet(3) and Frechet(5) distribution #
##########################################

RMSE.n.500.Frechet3 <- RMSE.n.500.Frechet5 <- RMSE.n.2000.Frechet3 <- RMSE.n.2000.Frechet5 <- RMSE.n.5000.Frechet3 <- RMSE.n.5000.Frechet5 <- matrix(NA, nrow=8, ncol=2)

RMSE.n.500.Frechet3[1, ]  <- MSE.90[6, 1:2]
RMSE.n.2000.Frechet3[1, ] <- MSE.90[6, 5:6]
RMSE.n.5000.Frechet3[1, ] <- MSE.90[6, 7:8]

RMSE.n.500.Frechet3[2, ]  <- MSE.95[6, 1:2]
RMSE.n.2000.Frechet3[2, ] <- MSE.95[6, 5:6]
RMSE.n.5000.Frechet3[2, ] <- MSE.95[6, 7:8]

RMSE.n.500.Frechet3[3, ]  <- MSE.975[6, 1:2]
RMSE.n.2000.Frechet3[3, ] <- MSE.975[6, 5:6]
RMSE.n.5000.Frechet3[3, ] <- MSE.975[6, 7:8]

RMSE.n.500.Frechet3[4, ]  <- MSE.99[6, 1:2]
RMSE.n.2000.Frechet3[4, ] <- MSE.99[6, 5:6]
RMSE.n.5000.Frechet3[4, ] <- MSE.99[6, 7:8]

RMSE.n.500.Frechet3[5, ]  <- MSE.99.5[6, 1:2]
RMSE.n.2000.Frechet3[5, ] <- MSE.99.5[6, 5:6]
RMSE.n.5000.Frechet3[5, ] <- MSE.99.5[6, 7:8]

RMSE.n.500.Frechet3[6, ]  <- MSE.99.9[6, 1:2]
RMSE.n.2000.Frechet3[6, ] <- MSE.99.9[6, 5:6]
RMSE.n.5000.Frechet3[6, ] <- MSE.99.9[6, 7:8]

RMSE.n.500.Frechet3[7, ]  <- MSE.99.95[6, 1:2]
RMSE.n.2000.Frechet3[7, ] <- MSE.99.95[6, 5:6]
RMSE.n.5000.Frechet3[7, ] <- MSE.99.95[6, 7:8]

RMSE.n.500.Frechet3[8, ]  <- MSE.99.99[6, 1:2]
RMSE.n.2000.Frechet3[8, ] <- MSE.99.99[6, 5:6]
RMSE.n.5000.Frechet3[8, ] <- MSE.99.99[6, 7:8]


RMSE.n.500.Frechet5[1, ]  <- MSE.90[7, 1:2]
RMSE.n.2000.Frechet5[1, ] <- MSE.90[7, 5:6]
RMSE.n.5000.Frechet5[1, ] <- MSE.90[7, 7:8]

RMSE.n.500.Frechet5[2, ]  <- MSE.95[7, 1:2]
RMSE.n.2000.Frechet5[2, ] <- MSE.95[7, 5:6]
RMSE.n.5000.Frechet5[2, ] <- MSE.95[7, 7:8]

RMSE.n.500.Frechet5[3, ]  <- MSE.975[7, 1:2]
RMSE.n.2000.Frechet5[3, ] <- MSE.975[7, 5:6]
RMSE.n.5000.Frechet5[3, ] <- MSE.975[7, 7:8]

RMSE.n.500.Frechet5[4, ]  <- MSE.99[7, 1:2]
RMSE.n.2000.Frechet5[4, ] <- MSE.99[7, 5:6]
RMSE.n.5000.Frechet5[4, ] <- MSE.99[7, 7:8]

RMSE.n.500.Frechet5[5, ]  <- MSE.99.5[7, 1:2]
RMSE.n.2000.Frechet5[5, ] <- MSE.99.5[7, 5:6]
RMSE.n.5000.Frechet5[5, ] <- MSE.99.5[7, 7:8]

RMSE.n.500.Frechet5[6, ]  <- MSE.99.9[7, 1:2]
RMSE.n.2000.Frechet5[6, ] <- MSE.99.9[7, 5:6]
RMSE.n.5000.Frechet5[6, ] <- MSE.99.9[7, 7:8]

RMSE.n.500.Frechet5[7, ]  <- MSE.99.95[7, 1:2]
RMSE.n.2000.Frechet5[7, ] <- MSE.99.95[7, 5:6]
RMSE.n.5000.Frechet5[7, ] <- MSE.99.95[7, 7:8]

RMSE.n.500.Frechet5[8, ]  <- MSE.99.99[7, 1:2]
RMSE.n.2000.Frechet5[8, ] <- MSE.99.99[7, 5:6]
RMSE.n.5000.Frechet5[8, ] <- MSE.99.99[7, 7:8]


##############################
# ARCH 2 and 4 distribution #
##############################

RMSE.n.500.GARCH2 <- RMSE.n.500.GARCH4 <- RMSE.n.2000.GARCH2 <- RMSE.n.2000.GARCH4 <- RMSE.n.5000.GARCH2 <- RMSE.n.5000.GARCH4 <- matrix(NA, nrow=8, ncol=2)

RMSE.n.500.GARCH2[1, ]  <- MSE.90[21, 1:2]
RMSE.n.2000.GARCH2[1, ] <- MSE.90[21, 5:6]
RMSE.n.5000.GARCH2[1, ] <- MSE.90[21, 7:8]

RMSE.n.500.GARCH2[2, ]  <- MSE.95[21, 1:2]
RMSE.n.2000.GARCH2[2, ] <- MSE.95[21, 5:6]
RMSE.n.5000.GARCH2[2, ] <- MSE.95[21, 7:8]

RMSE.n.500.GARCH2[3, ]  <- MSE.975[21, 1:2]
RMSE.n.2000.GARCH2[3, ] <- MSE.975[21, 5:6]
RMSE.n.5000.GARCH2[3, ] <- MSE.975[21, 7:8]

RMSE.n.500.GARCH2[4, ]  <- MSE.99[21, 1:2]
RMSE.n.2000.GARCH2[4, ] <- MSE.99[21, 5:6]
RMSE.n.5000.GARCH2[4, ] <- MSE.99[21, 7:8]

RMSE.n.500.GARCH2[5, ]  <- MSE.99.5[21, 1:2]
RMSE.n.2000.GARCH2[5, ] <- MSE.99.5[21, 5:6]
RMSE.n.5000.GARCH2[5, ] <- MSE.99.5[21, 7:8]

RMSE.n.500.GARCH2[6, ]  <- MSE.99.9[21, 1:2]
RMSE.n.2000.GARCH2[6, ] <- MSE.99.9[21, 5:6]
RMSE.n.5000.GARCH2[6, ] <- MSE.99.9[21, 7:8]

RMSE.n.500.GARCH2[7, ]  <- MSE.99.95[21, 1:2]
RMSE.n.2000.GARCH2[7, ] <- MSE.99.95[21, 5:6]
RMSE.n.5000.GARCH2[7, ] <- MSE.99.95[21, 7:8]

RMSE.n.500.GARCH2[8, ]  <- MSE.99.99[21, 1:2]
RMSE.n.2000.GARCH2[8, ] <- MSE.99.99[21, 5:6]
RMSE.n.5000.GARCH2[8, ] <- MSE.99.99[21, 7:8]


RMSE.n.500.GARCH4[1, ]  <- MSE.90[24, 1:2]
RMSE.n.2000.GARCH4[1, ] <- MSE.90[24, 5:6]
RMSE.n.5000.GARCH4[1, ] <- MSE.90[24, 7:8]

RMSE.n.500.GARCH4[2, ]  <- MSE.95[24, 1:2]
RMSE.n.2000.GARCH4[2, ] <- MSE.95[24, 5:6]
RMSE.n.5000.GARCH4[2, ] <- MSE.95[24, 7:8]

RMSE.n.500.GARCH4[3, ]  <- MSE.975[24, 1:2]
RMSE.n.2000.GARCH4[3, ] <- MSE.975[24, 5:6]
RMSE.n.5000.GARCH4[3, ] <- MSE.975[24, 7:8]

RMSE.n.500.GARCH4[4, ]  <- MSE.99[24, 1:2]
RMSE.n.2000.GARCH4[4, ] <- MSE.99[24, 5:6]
RMSE.n.5000.GARCH4[4, ] <- MSE.99[24, 7:8]

RMSE.n.500.GARCH4[5, ]  <- MSE.99.5[24, 1:2]
RMSE.n.2000.GARCH4[5, ] <- MSE.99.5[24, 5:6]
RMSE.n.5000.GARCH4[5, ] <- MSE.99.5[24, 7:8]

RMSE.n.500.GARCH4[6, ]  <- MSE.99.9[24, 1:2]
RMSE.n.2000.GARCH4[6, ] <- MSE.99.9[24, 5:6]
RMSE.n.5000.GARCH4[6, ] <- MSE.99.9[24, 7:8]

RMSE.n.500.GARCH4[7, ]  <- MSE.99.95[24, 1:2]
RMSE.n.2000.GARCH4[7, ] <- MSE.99.95[24, 5:6]
RMSE.n.5000.GARCH4[7, ] <- MSE.99.95[24, 7:8]

RMSE.n.500.GARCH4[8, ]  <- MSE.99.99[24, 1:2]
RMSE.n.2000.GARCH4[8, ] <- MSE.99.99[24, 5:6]
RMSE.n.5000.GARCH4[8, ] <- MSE.99.99[24, 7:8]


#################
# Plot Figure 1 #
#################

par(mfrow=c(3, 2))

alpha     = c(0.9, 0.95, 0.975, 0.99, 0.995, 0.999, 0.9995, 0.9999)
ticks = 1-alpha

# 1. model
options(scipen=999)
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.n.500.t3[ , 1] / RMSE.n.500.t3[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.n.2000.t3[ , 1] / RMSE.n.2000.t3[ , 2], lty=2)
lines(1-alpha, RMSE.n.5000.t3[ , 1] / RMSE.n.5000.t3[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(a1) "~italic(t)~"-distribution ("~alpha ~"= 3 )"), side=1, line = 4)


# 2. model
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.n.500.t5[ , 1] / RMSE.n.500.t5[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.n.2000.t5[ , 1] / RMSE.n.2000.t5[ , 2], lty=2)
lines(1-alpha, RMSE.n.5000.t5 [ , 1] / RMSE.n.5000.t5[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(a2) "~italic(t)~"-distribution ("~alpha ~"= 5 )"), side=1, line = 4)


# 3. model
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.n.500.Frechet3[ , 1] / RMSE.n.500.Frechet3[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.n.2000.Frechet3[ , 1] / RMSE.n.2000.Frechet3[ , 2], lty=2)
lines(1-alpha, RMSE.n.5000.Frechet3[ , 1] / RMSE.n.5000.Frechet3[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(b1) Frechet-distribution ("~alpha ~"= 3 )"), side=1, line = 4)


# 4. model
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.n.500.Frechet5[ , 1] / RMSE.n.500.Frechet5[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.n.2000.Frechet5[ , 1] / RMSE.n.2000.Frechet5[ , 2], lty=2)
lines(1-alpha, RMSE.n.5000.Frechet5[ , 1] / RMSE.n.5000.Frechet5[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(b2) Frechet-distribution ("~alpha ~"= 5 )"), side=1, line = 4)


# 5. model
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.n.500.GARCH2[ , 1] / RMSE.n.500.GARCH2[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.n.2000.GARCH2[ , 1] / RMSE.n.2000.GARCH2[ , 2], lty=2)
lines(1-alpha, RMSE.n.5000.GARCH2[ , 1] / RMSE.n.5000.GARCH2[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(c1) ARCH ("~alpha ~"= 2.30 )"), side=1, line = 4)


# 6. model
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.n.500.GARCH4[ , 1] / RMSE.n.500.GARCH4[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.n.2000.GARCH4[ , 1] / RMSE.n.2000.GARCH4[ , 2], lty=2)
lines(1-alpha, RMSE.n.5000.GARCH4[ , 1] / RMSE.n.5000.GARCH4[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(c2) ARCH ("~alpha ~"= 3.82 )"), side=1, line = 4)



# ===================================================
# Figure A.1: RMSE ratios for ES estimation for (M1), (M2) and (M6)
# ===================================================

############################
# t_3 and t_5 distribution #
############################

RMSE.ES.n.500.t3 <- RMSE.ES.n.500.t5 <- RMSE.ES.n.2000.t3 <- RMSE.ES.n.2000.t5 <- RMSE.ES.n.5000.t3 <- RMSE.ES.n.5000.t5 <- matrix(NA, nrow=8, ncol=2)

RMSE.ES.n.500.t3[1, ]  <- MSE.ES.90[2, 1:2]
RMSE.ES.n.2000.t3[1, ] <- MSE.ES.90[2, 5:6]
RMSE.ES.n.5000.t3[1, ] <- MSE.ES.90[2, 7:8]
RMSE.ES.n.500.t3[2, ]  <- MSE.ES.95[2, 1:2]
RMSE.ES.n.2000.t3[2, ] <- MSE.ES.95[2, 5:6]
RMSE.ES.n.5000.t3[2, ] <- MSE.ES.95[2, 7:8]
RMSE.ES.n.500.t3[3, ]  <- MSE.ES.975[2, 1:2]
RMSE.ES.n.2000.t3[3, ] <- MSE.ES.975[2, 5:6]
RMSE.ES.n.5000.t3[3, ] <- MSE.ES.975[2, 7:8]
RMSE.ES.n.500.t3[4, ]  <- MSE.ES.99[2, 1:2]
RMSE.ES.n.2000.t3[4, ] <- MSE.ES.99[2, 5:6]
RMSE.ES.n.5000.t3[4, ] <- MSE.ES.99[2, 7:8]
RMSE.ES.n.500.t3[5, ]  <- MSE.ES.99.5[2, 1:2]
RMSE.ES.n.2000.t3[5, ] <- MSE.ES.99.5[2, 5:6]
RMSE.ES.n.5000.t3[5, ] <- MSE.ES.99.5[2, 7:8]
RMSE.ES.n.500.t3[6, ]  <- MSE.ES.99.9[2, 1:2]
RMSE.ES.n.2000.t3[6, ] <- MSE.ES.99.9[2, 5:6]
RMSE.ES.n.5000.t3[6, ] <- MSE.ES.99.9[2, 7:8]
RMSE.ES.n.500.t3[7, ]  <- MSE.ES.99.95[2, 1:2]
RMSE.ES.n.2000.t3[7, ] <- MSE.ES.99.95[2, 5:6]
RMSE.ES.n.5000.t3[7, ] <- MSE.ES.99.95[2, 7:8]
RMSE.ES.n.500.t3[8, ]  <- MSE.ES.99.99[2, 1:2]
RMSE.ES.n.2000.t3[8, ] <- MSE.ES.99.99[2, 5:6]
RMSE.ES.n.5000.t3[8, ] <- MSE.ES.99.99[2, 7:8]


RMSE.ES.n.500.t5[1, ]  <- MSE.ES.90[3, 1:2]
RMSE.ES.n.2000.t5[1, ] <- MSE.ES.90[3, 5:6]
RMSE.ES.n.5000.t5[1, ] <- MSE.ES.90[3, 7:8]

RMSE.ES.n.500.t5[2, ]  <- MSE.ES.95[3, 1:2]
RMSE.ES.n.2000.t5[2, ] <- MSE.ES.95[3, 5:6]
RMSE.ES.n.5000.t5[2, ] <- MSE.ES.95[3, 7:8]

RMSE.ES.n.500.t5[3, ]  <- MSE.ES.975[3, 1:2]
RMSE.ES.n.2000.t5[3, ] <- MSE.ES.975[3, 5:6]
RMSE.ES.n.5000.t5[3, ] <- MSE.ES.975[3, 7:8]

RMSE.ES.n.500.t5[4, ]  <- MSE.ES.99[3, 1:2]
RMSE.ES.n.2000.t5[4, ] <- MSE.ES.99[3, 5:6]
RMSE.ES.n.5000.t5[4, ] <- MSE.ES.99[3, 7:8]

RMSE.ES.n.500.t5[5, ]  <- MSE.ES.99.5[3, 1:2]
RMSE.ES.n.2000.t5[5, ] <- MSE.ES.99.5[3, 5:6]
RMSE.ES.n.5000.t5[5, ] <- MSE.ES.99.5[3, 7:8]

RMSE.ES.n.500.t5[6, ]  <- MSE.ES.99.9[3, 1:2]
RMSE.ES.n.2000.t5[6, ] <- MSE.ES.99.9[3, 5:6]
RMSE.ES.n.5000.t5[6, ] <- MSE.ES.99.9[3, 7:8]

RMSE.ES.n.500.t5[7, ]  <- MSE.ES.99.95[3, 1:2]
RMSE.ES.n.2000.t5[7, ] <- MSE.ES.99.95[3, 5:6]
RMSE.ES.n.5000.t5[7, ] <- MSE.ES.99.95[3, 7:8]

RMSE.ES.n.500.t5[8, ]  <- MSE.ES.99.99[3, 1:2]
RMSE.ES.n.2000.t5[8, ] <- MSE.ES.99.99[3, 5:6]
RMSE.ES.n.5000.t5[8, ] <- MSE.ES.99.99[3, 7:8]

##########################################
# Frechet(3) and Frechet(5) distribution #
##########################################

RMSE.ES.n.500.Frechet3 <- RMSE.ES.n.500.Frechet5 <- RMSE.ES.n.2000.Frechet3 <- RMSE.ES.n.2000.Frechet5 <- RMSE.ES.n.5000.Frechet3 <- RMSE.ES.n.5000.Frechet5 <- matrix(NA, nrow=8, ncol=2)

RMSE.ES.n.500.Frechet3[1, ]  <- MSE.ES.90[6, 1:2]
RMSE.ES.n.2000.Frechet3[1, ] <- MSE.ES.90[6, 5:6]
RMSE.ES.n.5000.Frechet3[1, ] <- MSE.ES.90[6, 7:8]

RMSE.ES.n.500.Frechet3[2, ]  <- MSE.ES.95[6, 1:2]
RMSE.ES.n.2000.Frechet3[2, ] <- MSE.ES.95[6, 5:6]
RMSE.ES.n.5000.Frechet3[2, ] <- MSE.ES.95[6, 7:8]

RMSE.ES.n.500.Frechet3[3, ]  <- MSE.ES.975[6, 1:2]
RMSE.ES.n.2000.Frechet3[3, ] <- MSE.ES.975[6, 5:6]
RMSE.ES.n.5000.Frechet3[3, ] <- MSE.ES.975[6, 7:8]

RMSE.ES.n.500.Frechet3[4, ]  <- MSE.ES.99[6, 1:2]
RMSE.ES.n.2000.Frechet3[4, ] <- MSE.ES.99[6, 5:6]
RMSE.ES.n.5000.Frechet3[4, ] <- MSE.ES.99[6, 7:8]

RMSE.ES.n.500.Frechet3[5, ]  <- MSE.ES.99.5[6, 1:2]
RMSE.ES.n.2000.Frechet3[5, ] <- MSE.ES.99.5[6, 5:6]
RMSE.ES.n.5000.Frechet3[5, ] <- MSE.ES.99.5[6, 7:8]

RMSE.ES.n.500.Frechet3[6, ]  <- MSE.ES.99.9[6, 1:2]
RMSE.ES.n.2000.Frechet3[6, ] <- MSE.ES.99.9[6, 5:6]
RMSE.ES.n.5000.Frechet3[6, ] <- MSE.ES.99.9[6, 7:8]

RMSE.ES.n.500.Frechet3[7, ]  <- MSE.ES.99.95[6, 1:2]
RMSE.ES.n.2000.Frechet3[7, ] <- MSE.ES.99.95[6, 5:6]
RMSE.ES.n.5000.Frechet3[7, ] <- MSE.ES.99.95[6, 7:8]

RMSE.ES.n.500.Frechet3[8, ]  <- MSE.ES.99.99[6, 1:2]
RMSE.ES.n.2000.Frechet3[8, ] <- MSE.ES.99.99[6, 5:6]
RMSE.ES.n.5000.Frechet3[8, ] <- MSE.ES.99.99[6, 7:8]


RMSE.ES.n.500.Frechet5[1, ]  <- MSE.ES.90[7, 1:2]
RMSE.ES.n.2000.Frechet5[1, ] <- MSE.ES.90[7, 5:6]
RMSE.ES.n.5000.Frechet5[1, ] <- MSE.ES.90[7, 7:8]

RMSE.ES.n.500.Frechet5[2, ]  <- MSE.ES.95[7, 1:2]
RMSE.ES.n.2000.Frechet5[2, ] <- MSE.ES.95[7, 5:6]
RMSE.ES.n.5000.Frechet5[2, ] <- MSE.ES.95[7, 7:8]

RMSE.ES.n.500.Frechet5[3, ]  <- MSE.ES.975[7, 1:2]
RMSE.ES.n.2000.Frechet5[3, ] <- MSE.ES.975[7, 5:6]
RMSE.ES.n.5000.Frechet5[3, ] <- MSE.ES.975[7, 7:8]

RMSE.ES.n.500.Frechet5[4, ]  <- MSE.ES.99[7, 1:2]
RMSE.ES.n.2000.Frechet5[4, ] <- MSE.ES.99[7, 5:6]
RMSE.ES.n.5000.Frechet5[4, ] <- MSE.ES.99[7, 7:8]

RMSE.ES.n.500.Frechet5[5, ]  <- MSE.ES.99.5[7, 1:2]
RMSE.ES.n.2000.Frechet5[5, ] <- MSE.ES.99.5[7, 5:6]
RMSE.ES.n.5000.Frechet5[5, ] <- MSE.ES.99.5[7, 7:8]

RMSE.ES.n.500.Frechet5[6, ]  <- MSE.ES.99.9[7, 1:2]
RMSE.ES.n.2000.Frechet5[6, ] <- MSE.ES.99.9[7, 5:6]
RMSE.ES.n.5000.Frechet5[6, ] <- MSE.ES.99.9[7, 7:8]

RMSE.ES.n.500.Frechet5[7, ]  <- MSE.ES.99.95[7, 1:2]
RMSE.ES.n.2000.Frechet5[7, ] <- MSE.ES.99.95[7, 5:6]
RMSE.ES.n.5000.Frechet5[7, ] <- MSE.ES.99.95[7, 7:8]

RMSE.ES.n.500.Frechet5[8, ]  <- MSE.ES.99.99[7, 1:2]
RMSE.ES.n.2000.Frechet5[8, ] <- MSE.ES.99.99[7, 5:6]
RMSE.ES.n.5000.Frechet5[8, ] <- MSE.ES.99.99[7, 7:8]


##############################
# ARCH 2 and 4 distribution #
##############################

RMSE.ES.n.500.GARCH2 <- RMSE.ES.n.500.GARCH4 <- RMSE.ES.n.2000.GARCH2 <- RMSE.ES.n.2000.GARCH4 <- RMSE.ES.n.5000.GARCH2 <- RMSE.ES.n.5000.GARCH4 <- matrix(NA, nrow=8, ncol=2)

RMSE.ES.n.500.GARCH2[1, ]  <- MSE.ES.90[21, 1:2]
RMSE.ES.n.2000.GARCH2[1, ] <- MSE.ES.90[21, 5:6]
RMSE.ES.n.5000.GARCH2[1, ] <- MSE.ES.90[21, 7:8]

RMSE.ES.n.500.GARCH2[2, ]  <- MSE.ES.95[21, 1:2]
RMSE.ES.n.2000.GARCH2[2, ] <- MSE.ES.95[21, 5:6]
RMSE.ES.n.5000.GARCH2[2, ] <- MSE.ES.95[21, 7:8]

RMSE.ES.n.500.GARCH2[3, ]  <- MSE.ES.975[21, 1:2]
RMSE.ES.n.2000.GARCH2[3, ] <- MSE.ES.975[21, 5:6]
RMSE.ES.n.5000.GARCH2[3, ] <- MSE.ES.975[21, 7:8]

RMSE.ES.n.500.GARCH2[4, ]  <- MSE.ES.99[21, 1:2]
RMSE.ES.n.2000.GARCH2[4, ] <- MSE.ES.99[21, 5:6]
RMSE.ES.n.5000.GARCH2[4, ] <- MSE.ES.99[21, 7:8]

RMSE.ES.n.500.GARCH2[5, ]  <- MSE.ES.99.5[21, 1:2]
RMSE.ES.n.2000.GARCH2[5, ] <- MSE.ES.99.5[21, 5:6]
RMSE.ES.n.5000.GARCH2[5, ] <- MSE.ES.99.5[21, 7:8]

RMSE.ES.n.500.GARCH2[6, ]  <- MSE.ES.99.9[21, 1:2]
RMSE.ES.n.2000.GARCH2[6, ] <- MSE.ES.99.9[21, 5:6]
RMSE.ES.n.5000.GARCH2[6, ] <- MSE.ES.99.9[21, 7:8]

RMSE.ES.n.500.GARCH2[7, ]  <- MSE.ES.99.95[21, 1:2]
RMSE.ES.n.2000.GARCH2[7, ] <- MSE.ES.99.95[21, 5:6]
RMSE.ES.n.5000.GARCH2[7, ] <- MSE.ES.99.95[21, 7:8]

RMSE.ES.n.500.GARCH2[8, ]  <- MSE.ES.99.99[21, 1:2]
RMSE.ES.n.2000.GARCH2[8, ] <- MSE.ES.99.99[21, 5:6]
RMSE.ES.n.5000.GARCH2[8, ] <- MSE.ES.99.99[21, 7:8]


RMSE.ES.n.500.GARCH4[1, ]  <- MSE.ES.90[24, 1:2]
RMSE.ES.n.2000.GARCH4[1, ] <- MSE.ES.90[24, 5:6]
RMSE.ES.n.5000.GARCH4[1, ] <- MSE.ES.90[24, 7:8]

RMSE.ES.n.500.GARCH4[2, ]  <- MSE.ES.95[24, 1:2]
RMSE.ES.n.2000.GARCH4[2, ] <- MSE.ES.95[24, 5:6]
RMSE.ES.n.5000.GARCH4[2, ] <- MSE.ES.95[24, 7:8]

RMSE.ES.n.500.GARCH4[3, ]  <- MSE.ES.975[24, 1:2]
RMSE.ES.n.2000.GARCH4[3, ] <- MSE.ES.975[24, 5:6]
RMSE.ES.n.5000.GARCH4[3, ] <- MSE.ES.975[24, 7:8]

RMSE.ES.n.500.GARCH4[4, ]  <- MSE.ES.99[24, 1:2]
RMSE.ES.n.2000.GARCH4[4, ] <- MSE.ES.99[24, 5:6]
RMSE.ES.n.5000.GARCH4[4, ] <- MSE.ES.99[24, 7:8]

RMSE.ES.n.500.GARCH4[5, ]  <- MSE.ES.99.5[24, 1:2]
RMSE.ES.n.2000.GARCH4[5, ] <- MSE.ES.99.5[24, 5:6]
RMSE.ES.n.5000.GARCH4[5, ] <- MSE.ES.99.5[24, 7:8]

RMSE.ES.n.500.GARCH4[6, ]  <- MSE.ES.99.9[24, 1:2]
RMSE.ES.n.2000.GARCH4[6, ] <- MSE.ES.99.9[24, 5:6]
RMSE.ES.n.5000.GARCH4[6, ] <- MSE.ES.99.9[24, 7:8]

RMSE.ES.n.500.GARCH4[7, ]  <- MSE.ES.99.95[24, 1:2]
RMSE.ES.n.2000.GARCH4[7, ] <- MSE.ES.99.95[24, 5:6]
RMSE.ES.n.5000.GARCH4[7, ] <- MSE.ES.99.95[24, 7:8]

RMSE.ES.n.500.GARCH4[8, ]  <- MSE.ES.99.99[24, 1:2]
RMSE.ES.n.2000.GARCH4[8, ] <- MSE.ES.99.99[24, 5:6]
RMSE.ES.n.5000.GARCH4[8, ] <- MSE.ES.99.99[24, 7:8]


###################
# Plot Figure A.1 #
###################

par(mfrow=c(3, 2))

alpha     = c(0.9, 0.95, 0.975, 0.99, 0.995, 0.999, 0.9995, 0.9999)
ticks = 1-alpha

# 1. model
options(scipen=999)
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.ES.n.500.t3[ , 1] / RMSE.ES.n.500.t3[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.ES.n.2000.t3[ , 1] / RMSE.ES.n.2000.t3[ , 2], lty=2)
lines(1-alpha, RMSE.ES.n.5000.t3[ , 1] / RMSE.ES.n.5000.t3[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(a1) "~italic(t)~"-distribution ("~alpha ~"= 3 )"), side=1, line = 4)


# 2. model
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.ES.n.500.t5[ , 1] / RMSE.ES.n.500.t5[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.ES.n.2000.t5[ , 1] / RMSE.ES.n.2000.t5[ , 2], lty=2)
lines(1-alpha, RMSE.ES.n.5000.t5 [ , 1] / RMSE.ES.n.5000.t5[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(a2) "~italic(t)~"-distribution ("~alpha ~"= 5 )"), side=1, line = 4)


# 3. model
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.ES.n.500.Frechet3[ , 1] / RMSE.ES.n.500.Frechet3[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.ES.n.2000.Frechet3[ , 1] / RMSE.ES.n.2000.Frechet3[ , 2], lty=2)
lines(1-alpha, RMSE.ES.n.5000.Frechet3[ , 1] / RMSE.ES.n.5000.Frechet3[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(b1) Frechet-distribution ("~alpha ~"= 3 )"), side=1, line = 4)


# 4. model
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.ES.n.500.Frechet5[ , 1] / RMSE.ES.n.500.Frechet5[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.ES.n.2000.Frechet5[ , 1] / RMSE.ES.n.2000.Frechet5[ , 2], lty=2)
lines(1-alpha, RMSE.ES.n.5000.Frechet5[ , 1] / RMSE.ES.n.5000.Frechet5[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(b2) Frechet-distribution ("~alpha ~"= 5 )"), side=1, line = 4)


# 5. model
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.ES.n.500.GARCH2[ , 1] / RMSE.ES.n.500.GARCH2[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.ES.n.2000.GARCH2[ , 1] / RMSE.ES.n.2000.GARCH2[ , 2], lty=2)
lines(1-alpha, RMSE.ES.n.5000.GARCH2[ , 1] / RMSE.ES.n.5000.GARCH2[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(c1) ARCH ("~alpha ~"= 2.30 )"), side=1, line = 4)


# 6. model
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.ES.n.500.GARCH4[ , 1] / RMSE.ES.n.500.GARCH4[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.ES.n.2000.GARCH4[ , 1] / RMSE.ES.n.2000.GARCH4[ , 2], lty=2)
lines(1-alpha, RMSE.ES.n.5000.GARCH4[ , 1] / RMSE.ES.n.5000.GARCH4[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(c2) ARCH ("~alpha ~"= 3.82 )"), side=1, line = 4)



# ===================================================
# Figure 2: Kernel-density plots of Hill estimates
# ===================================================


par(mfrow=c(3, 2))

# 1. model
options(scipen=999)
par(mar=c(5.1, 4, 2, 2))
i=2  # t_{3} distribution
x <- seq(-3,3, by = 0.1)
plot(x, dnorm(x),          # kernel density plot of sqrt(k^{Q})( \widehat{\gamma}(k^{Q}) - \gamma )
     lty=3, xlim=c(-3, 3), ylim=c(0.0, 0.6), yaxs="i",
     xlab = "", ylab = "density", las=1, type="l", col="red")
# n=500
dens.500.FIX <- density( sqrt(res.n.500[[i]]$k.fix) / res.n.500[[i]]$gamma * (res.n.500[[i]]$gamma.fix - res.n.500[[i]]$gamma) )
dens.500.SCR <- density( sqrt(res.n.500[[i]]$k.all[, 2]) / res.n.500[[i]]$gamma * (res.n.500[[i]]$gamma.all[, 2] - res.n.500[[i]]$gamma) )

lines(dens.500.FIX, type="l", lty=1, col="blue")  # kernel density plot of sqrt(k^{Q})( \widehat{\gamma}(k^{Q}) - \gamma )
lines(dens.500.SCR, type="l", lty=1)  # kernel density plot of sqrt(k^{QCRPS})( \widehat{\gamma}(k^{QCRPS}) - \gamma )

sub.FIX <-  seq(1, length(dens.500.FIX$x), 40)
sub.SCR <-  seq(1, length(dens.500.SCR$x), 40)

points(dens.500.FIX$x[sub.FIX], dens.500.FIX$y[sub.FIX], pch=1)
points(dens.500.SCR$x[sub.SCR], dens.500.SCR$y[sub.SCR], pch=2)

# n=5000
dens.5000.FIX <- density( sqrt(res.n.5000[[i]]$k.fix) / res.n.5000[[i]]$gamma * (res.n.5000[[i]]$gamma.fix - res.n.5000[[i]]$gamma) )
dens.5000.SCR <- density( sqrt(res.n.5000[[i]]$k.all[, 2]) / res.n.5000[[i]]$gamma * (res.n.5000[[i]]$gamma.all[, 2] - res.n.5000[[i]]$gamma) )

lines(dens.5000.FIX, type="l", lty=2, col="blue")  # kernel density plot of sqrt(k^{Q})( \widehat{\gamma}(k^{Q}) - \gamma )
lines(dens.5000.SCR, type="l", lty=2)  # kernel density plot of sqrt(k^{QCRPS})( \widehat{\gamma}(k^{QCRPS}) - \gamma )

sub.MAX <-  seq(1, length(dens.5000.FIX$x), 40)
sub.SCR <-  seq(1, length(dens.5000.SCR$x), 40)

points(dens.5000.FIX$x[sub.MAX], dens.5000.FIX$y[sub.MAX], pch=1)
points(dens.5000.SCR$x[sub.SCR], dens.5000.SCR$y[sub.SCR], pch=2)

legend("topleft", legend=c(expression(bar(k)), expression(paste("k"^"QCRPS")), "n=500", "n=5000"),
       lty=c(0, 0, 1, 2), pch = c(1, 2, NA, NA), ncol=2)
mtext(expression("(a1) "~italic(t)~"-distribution ("~alpha ~"= 3 )"), side=1, line = 3)


# 2. model
par(mar=c(5.1,4,2,2))
i=3  # t_{5} distribution
x <- seq(-3,3, by = 0.1)
plot(x, dnorm(x),          # kernel density plot of sqrt(k^{Q})( \widehat{\gamma}(k^{Q}) - \gamma )
     lty=3, xlim=c(-3, 3), ylim=c(0.0, 0.6), yaxs="i",
     xlab = "", ylab = "density", las=1, type="l", col="red")
# n=500
dens.500.FIX <- density( sqrt(res.n.500[[i]]$k.fix) / res.n.500[[i]]$gamma * (res.n.500[[i]]$gamma.fix - res.n.500[[i]]$gamma) )
dens.500.SCR <- density( sqrt(res.n.500[[i]]$k.all[, 2]) / res.n.500[[i]]$gamma * (res.n.500[[i]]$gamma.all[, 2] - res.n.500[[i]]$gamma) )

lines(dens.500.FIX, type="l", lty=1, col="blue")  # kernel density plot of sqrt(k^{Q})( \widehat{\gamma}(k^{Q}) - \gamma )
lines(dens.500.SCR, type="l", lty=1)  # kernel density plot of sqrt(k^{QCRPS})( \widehat{\gamma}(k^{QCRPS}) - \gamma )

sub.FIX <-  seq(1, length(dens.500.FIX$x), 40)
sub.SCR <-  seq(1, length(dens.500.SCR$x), 40)

points(dens.500.FIX$x[sub.FIX], dens.500.FIX$y[sub.FIX], pch=1)
points(dens.500.SCR$x[sub.SCR], dens.500.SCR$y[sub.SCR], pch=2)

# n=5000
dens.5000.FIX <- density( sqrt(res.n.5000[[i]]$k.fix) / res.n.5000[[i]]$gamma * (res.n.5000[[i]]$gamma.fix - res.n.5000[[i]]$gamma) )
dens.5000.SCR <- density( sqrt(res.n.5000[[i]]$k.all[, 2]) / res.n.5000[[i]]$gamma * (res.n.5000[[i]]$gamma.all[, 2] - res.n.5000[[i]]$gamma) )

lines(dens.5000.FIX, type="l", lty=2, col="blue")  # kernel density plot of sqrt(k^{Q})( \widehat{\gamma}(k^{Q}) - \gamma )
lines(dens.5000.SCR, type="l", lty=2)  # kernel density plot of sqrt(k^{QCRPS})( \widehat{\gamma}(k^{QCRPS}) - \gamma )

sub.MAX <-  seq(1, length(dens.5000.FIX$x), 40)
sub.SCR <-  seq(1, length(dens.5000.SCR$x), 40)

points(dens.5000.FIX$x[sub.MAX], dens.5000.FIX$y[sub.MAX], pch=1)
points(dens.5000.SCR$x[sub.SCR], dens.5000.SCR$y[sub.SCR], pch=2)

legend("topleft", legend=c(expression(bar(k)), expression(paste("k"^"QCRPS")), "n=500", "n=5000"),
       lty=c(0, 0, 1, 2), pch = c(1, 2, NA, NA), ncol=2)
mtext(expression("(a2) "~italic(t)~"-distribution ("~alpha ~"= 5 )"), side=1, line = 3)


# 3. model
par(mar=c(5.1,4,2,2))
i=6  # Frechet_{3} distribution
x <- seq(-3,3, by = 0.1)
plot(x, dnorm(x),          # kernel density plot of sqrt(k^{Q})( \widehat{\gamma}(k^{Q}) - \gamma )
     lty=3, xlim=c(-3, 3), ylim=c(0.0, 0.6), yaxs="i",
     xlab = "", ylab = "density", las=1, type="l", col="red")
# n=500
dens.500.FIX <- density( sqrt(res.n.500[[i]]$k.fix) / res.n.500[[i]]$gamma * (res.n.500[[i]]$gamma.fix - res.n.500[[i]]$gamma) )
dens.500.SCR <- density( sqrt(res.n.500[[i]]$k.all[, 2]) / res.n.500[[i]]$gamma * (res.n.500[[i]]$gamma.all[, 2] - res.n.500[[i]]$gamma) )

lines(dens.500.FIX, type="l", lty=1, col="blue")  # kernel density plot of sqrt(k^{Q})( \widehat{\gamma}(k^{Q}) - \gamma )
lines(dens.500.SCR, type="l", lty=1)  # kernel density plot of sqrt(k^{QCRPS})( \widehat{\gamma}(k^{QCRPS}) - \gamma )

sub.FIX <-  seq(1, length(dens.500.FIX$x), 40)
sub.SCR <-  seq(1, length(dens.500.SCR$x), 40)

points(dens.500.FIX$x[sub.FIX], dens.500.FIX$y[sub.FIX], pch=1)
points(dens.500.SCR$x[sub.SCR], dens.500.SCR$y[sub.SCR], pch=2)

# n=5000
dens.5000.FIX <- density( sqrt(res.n.5000[[i]]$k.fix) / res.n.5000[[i]]$gamma * (res.n.5000[[i]]$gamma.fix - res.n.5000[[i]]$gamma) )
dens.5000.SCR <- density( sqrt(res.n.5000[[i]]$k.all[, 2]) / res.n.5000[[i]]$gamma * (res.n.5000[[i]]$gamma.all[, 2] - res.n.5000[[i]]$gamma) )

lines(dens.5000.FIX, type="l", lty=2, col="blue")  # kernel density plot of sqrt(k^{Q})( \widehat{\gamma}(k^{Q}) - \gamma )
lines(dens.5000.SCR, type="l", lty=2)  # kernel density plot of sqrt(k^{QCRPS})( \widehat{\gamma}(k^{QCRPS}) - \gamma )

sub.MAX <-  seq(1, length(dens.5000.FIX$x), 40)
sub.SCR <-  seq(1, length(dens.5000.SCR$x), 40)

points(dens.5000.FIX$x[sub.MAX], dens.5000.FIX$y[sub.MAX], pch=1)
points(dens.5000.SCR$x[sub.SCR], dens.5000.SCR$y[sub.SCR], pch=2)

legend("topleft", legend=c(expression(bar(k)), expression(paste("k"^"QCRPS")), "n=500", "n=5000"),
       lty=c(0, 0, 1, 2), pch = c(1, 2, NA, NA), ncol=2)
mtext(expression("(b1) Frechet-distribution ("~alpha ~"= 3 )"), side=1, line = 3)


# 4. model
par(mar=c(5.1,4,2,2))
i=7  # Frechet_{5} distribution
x <- seq(-3,3, by = 0.1)
plot(x, dnorm(x),          # kernel density plot of sqrt(k^{Q})( \widehat{\gamma}(k^{Q}) - \gamma )
     lty=3, xlim=c(-3, 3), ylim=c(0.0, 0.6), yaxs="i",
     xlab = "", ylab = "density", las=1, type="l", col="red")
# n=500
dens.500.FIX <- density( sqrt(res.n.500[[i]]$k.fix) / res.n.500[[i]]$gamma * (res.n.500[[i]]$gamma.fix - res.n.500[[i]]$gamma) )
dens.500.SCR <- density( sqrt(res.n.500[[i]]$k.all[, 2]) / res.n.500[[i]]$gamma * (res.n.500[[i]]$gamma.all[, 2] - res.n.500[[i]]$gamma) )

lines(dens.500.FIX, type="l", lty=1, col="blue")  # kernel density plot of sqrt(k^{Q})( \widehat{\gamma}(k^{Q}) - \gamma )
lines(dens.500.SCR, type="l", lty=1)  # kernel density plot of sqrt(k^{QCRPS})( \widehat{\gamma}(k^{QCRPS}) - \gamma )

sub.FIX <-  seq(1, length(dens.500.FIX$x), 40)
sub.SCR <-  seq(1, length(dens.500.SCR$x), 40)

points(dens.500.FIX$x[sub.FIX], dens.500.FIX$y[sub.FIX], pch=1)
points(dens.500.SCR$x[sub.SCR], dens.500.SCR$y[sub.SCR], pch=2)

# n=5000
dens.5000.FIX <- density( sqrt(res.n.5000[[i]]$k.fix) / res.n.5000[[i]]$gamma * (res.n.5000[[i]]$gamma.fix - res.n.5000[[i]]$gamma) )
dens.5000.SCR <- density( sqrt(res.n.5000[[i]]$k.all[, 2]) / res.n.5000[[i]]$gamma * (res.n.5000[[i]]$gamma.all[, 2] - res.n.5000[[i]]$gamma) )

lines(dens.5000.FIX, type="l", lty=2, col="blue")  # kernel density plot of sqrt(k^{Q})( \widehat{\gamma}(k^{Q}) - \gamma )
lines(dens.5000.SCR, type="l", lty=2)  # kernel density plot of sqrt(k^{QCRPS})( \widehat{\gamma}(k^{QCRPS}) - \gamma )

sub.MAX <-  seq(1, length(dens.5000.FIX$x), 40)
sub.SCR <-  seq(1, length(dens.5000.SCR$x), 40)

points(dens.5000.FIX$x[sub.MAX], dens.5000.FIX$y[sub.MAX], pch=1)
points(dens.5000.SCR$x[sub.SCR], dens.5000.SCR$y[sub.SCR], pch=2)

legend("topleft", legend=c(expression(bar(k)), expression(paste("k"^"QCRPS")), "n=500", "n=5000"),
       lty=c(0, 0, 1, 2), pch = c(1, 2, NA, NA), ncol=2)
mtext(expression("(b2) Frechet-distribution ("~alpha ~"= 5 )"), side=1, line = 3)


# 5. model
par(mar=c(5.1,4,2,2))
i=21  # ARCH_{2.30} distribution
x <- seq(-3,3, by = 0.1)
plot(x, dnorm(x),          # kernel density plot of sqrt(k^{Q})( \widehat{\gamma}(k^{Q}) - \gamma )
     lty=3, xlim=c(-3, 3), ylim=c(0.0, 0.6), yaxs="i",
     xlab = "", ylab = "density", las=1, type="l", col="red")
# n=500
dens.500.FIX <- density( sqrt(res.n.500[[i]]$k.fix) / res.n.500[[i]]$gamma * (res.n.500[[i]]$gamma.fix - res.n.500[[i]]$gamma) )
dens.500.SCR <- density( sqrt(res.n.500[[i]]$k.all[, 2]) / res.n.500[[i]]$gamma * (res.n.500[[i]]$gamma.all[, 2] - res.n.500[[i]]$gamma) )

lines(dens.500.FIX, type="l", lty=1, col="blue")  # kernel density plot of sqrt(k^{Q})( \widehat{\gamma}(k^{Q}) - \gamma )
lines(dens.500.SCR, type="l", lty=1)  # kernel density plot of sqrt(k^{QCRPS})( \widehat{\gamma}(k^{QCRPS}) - \gamma )

sub.FIX <-  seq(1, length(dens.500.FIX$x), 40)
sub.SCR <-  seq(1, length(dens.500.SCR$x), 40)

points(dens.500.FIX$x[sub.FIX], dens.500.FIX$y[sub.FIX], pch=1)
points(dens.500.SCR$x[sub.SCR], dens.500.SCR$y[sub.SCR], pch=2)

# n=5000
dens.5000.FIX <- density( sqrt(res.n.5000[[i]]$k.fix) / res.n.5000[[i]]$gamma * (res.n.5000[[i]]$gamma.fix - res.n.5000[[i]]$gamma) )
dens.5000.SCR <- density( sqrt(res.n.5000[[i]]$k.all[, 2]) / res.n.5000[[i]]$gamma * (res.n.5000[[i]]$gamma.all[, 2] - res.n.5000[[i]]$gamma) )

lines(dens.5000.FIX, type="l", lty=2, col="blue")  # kernel density plot of sqrt(k^{Q})( \widehat{\gamma}(k^{Q}) - \gamma )
lines(dens.5000.SCR, type="l", lty=2)  # kernel density plot of sqrt(k^{QCRPS})( \widehat{\gamma}(k^{QCRPS}) - \gamma )

sub.MAX <-  seq(1, length(dens.5000.FIX$x), 40)
sub.SCR <-  seq(1, length(dens.5000.SCR$x), 40)

points(dens.5000.FIX$x[sub.MAX], dens.5000.FIX$y[sub.MAX], pch=1)
points(dens.5000.SCR$x[sub.SCR], dens.5000.SCR$y[sub.SCR], pch=2)

legend("topleft", legend=c(expression(bar(k)), expression(paste("k"^"QCRPS")), "n=500", "n=5000"),
       lty=c(0, 0, 1, 2), pch = c(1, 2, NA, NA), ncol=2)
mtext(expression("(c1) ARCH ("~alpha ~"= 2.30 )"), side=1, line = 3)


# 6. model
par(mar=c(5.1,4,2,2))
i=24  # ARCH_{3.82} distribution
x <- seq(-3,3, by = 0.1)
plot(x, dnorm(x),          # kernel density plot of sqrt(k^{Q})( \widehat{\gamma}(k^{Q}) - \gamma )
     lty=3, xlim=c(-3, 3), ylim=c(0.0, 0.6), yaxs="i",
     xlab = "", ylab = "density", las=1, type="l", col="red")
# n=500
dens.500.FIX <- density( sqrt(res.n.500[[i]]$k.fix) / res.n.500[[i]]$gamma * (res.n.500[[i]]$gamma.fix - res.n.500[[i]]$gamma) )
dens.500.SCR <- density( sqrt(res.n.500[[i]]$k.all[, 2]) / res.n.500[[i]]$gamma * (res.n.500[[i]]$gamma.all[, 2] - res.n.500[[i]]$gamma) )

lines(dens.500.FIX, type="l", lty=1, col="blue")  # kernel density plot of sqrt(k^{Q})( \widehat{\gamma}(k^{Q}) - \gamma )
lines(dens.500.SCR, type="l", lty=1)  # kernel density plot of sqrt(k^{QCRPS})( \widehat{\gamma}(k^{QCRPS}) - \gamma )

sub.FIX <-  seq(1, length(dens.500.FIX$x), 40)
sub.SCR <-  seq(1, length(dens.500.SCR$x), 40)

points(dens.500.FIX$x[sub.FIX], dens.500.FIX$y[sub.FIX], pch=1)
points(dens.500.SCR$x[sub.SCR], dens.500.SCR$y[sub.SCR], pch=2)

# n=5000
dens.5000.FIX <- density( sqrt(res.n.5000[[i]]$k.fix) / res.n.5000[[i]]$gamma * (res.n.5000[[i]]$gamma.fix - res.n.5000[[i]]$gamma) )
dens.5000.SCR <- density( sqrt(res.n.5000[[i]]$k.all[, 2]) / res.n.5000[[i]]$gamma * (res.n.5000[[i]]$gamma.all[, 2] - res.n.5000[[i]]$gamma) )

lines(dens.5000.FIX, type="l", lty=2, col="blue")  # kernel density plot of sqrt(k^{Q})( \widehat{\gamma}(k^{Q}) - \gamma )
lines(dens.5000.SCR, type="l", lty=2)  # kernel density plot of sqrt(k^{QCRPS})( \widehat{\gamma}(k^{QCRPS}) - \gamma )

sub.MAX <-  seq(1, length(dens.5000.FIX$x), 40)
sub.SCR <-  seq(1, length(dens.5000.SCR$x), 40)

points(dens.5000.FIX$x[sub.MAX], dens.5000.FIX$y[sub.MAX], pch=1)
points(dens.5000.SCR$x[sub.SCR], dens.5000.SCR$y[sub.SCR], pch=2)

legend("topleft", legend=c(expression(bar(k)), expression(paste("k"^"QCRPS")), "n=500", "n=5000"),
       lty=c(0, 0, 1, 2), pch = c(1, 2, NA, NA), ncol=2)
mtext(expression("(c2) ARCH ("~alpha ~"= 3.82 )"), side=1, line = 3)





# ===================================================
# Figure A.2: RMSE ratios for VaR estimation for (M3)-(M5)
# ===================================================


################################
# SS_1 and SS_1.9 distribution #
################################

RMSE.n.500.SS1 <- RMSE.n.500.SS2 <- RMSE.n.2000.SS1 <- RMSE.n.2000.SS2 <- RMSE.n.5000.SS1 <- RMSE.n.5000.SS2 <- matrix(NA, nrow=8, ncol=2)

RMSE.n.500.SS1[1, ]  <- MSE.90[10, 1:2]
RMSE.n.2000.SS1[1, ] <- MSE.90[10, 5:6]
RMSE.n.5000.SS1[1, ] <- MSE.90[10, 7:8]
RMSE.n.500.SS1[2, ]  <- MSE.95[10, 1:2]
RMSE.n.2000.SS1[2, ] <- MSE.95[10, 5:6]
RMSE.n.5000.SS1[2, ] <- MSE.95[10, 7:8]
RMSE.n.500.SS1[3, ]  <- MSE.975[10, 1:2]
RMSE.n.2000.SS1[3, ] <- MSE.975[10, 5:6]
RMSE.n.5000.SS1[3, ] <- MSE.975[10, 7:8]
RMSE.n.500.SS1[4, ]  <- MSE.99[10, 1:2]
RMSE.n.2000.SS1[4, ] <- MSE.99[10, 5:6]
RMSE.n.5000.SS1[4, ] <- MSE.99[10, 7:8]
RMSE.n.500.SS1[5, ]  <- MSE.99.5[10, 1:2]
RMSE.n.2000.SS1[5, ] <- MSE.99.5[10, 5:6]
RMSE.n.5000.SS1[5, ] <- MSE.99.5[10, 7:8]
RMSE.n.500.SS1[6, ]  <- MSE.99.9[10, 1:2]
RMSE.n.2000.SS1[6, ] <- MSE.99.9[10, 5:6]
RMSE.n.5000.SS1[6, ] <- MSE.99.9[10, 7:8]
RMSE.n.500.SS1[7, ]  <- MSE.99.95[10, 1:2]
RMSE.n.2000.SS1[7, ] <- MSE.99.95[10, 5:6]
RMSE.n.5000.SS1[7, ] <- MSE.99.95[10, 7:8]
RMSE.n.500.SS1[8, ]  <- MSE.99.99[10, 1:2]
RMSE.n.2000.SS1[8, ] <- MSE.99.99[10, 5:6]
RMSE.n.5000.SS1[8, ] <- MSE.99.99[10, 7:8]


RMSE.n.500.SS2[1, ]  <- MSE.90[12, 1:2]
RMSE.n.2000.SS2[1, ] <- MSE.90[12, 5:6]
RMSE.n.5000.SS2[1, ] <- MSE.90[12, 7:8]

RMSE.n.500.SS2[2, ]  <- MSE.95[12, 1:2]
RMSE.n.2000.SS2[2, ] <- MSE.95[12, 5:6]
RMSE.n.5000.SS2[2, ] <- MSE.95[12, 7:8]

RMSE.n.500.SS2[3, ]  <- MSE.975[12, 1:2]
RMSE.n.2000.SS2[3, ] <- MSE.975[12, 5:6]
RMSE.n.5000.SS2[3, ] <- MSE.975[12, 7:8]

RMSE.n.500.SS2[4, ]  <- MSE.99[12, 1:2]
RMSE.n.2000.SS2[4, ] <- MSE.99[12, 5:6]
RMSE.n.5000.SS2[4, ] <- MSE.99[12, 7:8]

RMSE.n.500.SS2[5, ]  <- MSE.99.5[12, 1:2]
RMSE.n.2000.SS2[5, ] <- MSE.99.5[12, 5:6]
RMSE.n.5000.SS2[5, ] <- MSE.99.5[12, 7:8]

RMSE.n.500.SS2[6, ]  <- MSE.99.9[12, 1:2]
RMSE.n.2000.SS2[6, ] <- MSE.99.9[12, 5:6]
RMSE.n.5000.SS2[6, ] <- MSE.99.9[12, 7:8]

RMSE.n.500.SS2[7, ]  <- MSE.99.95[12, 1:2]
RMSE.n.2000.SS2[7, ] <- MSE.99.95[12, 5:6]
RMSE.n.5000.SS2[7, ] <- MSE.99.95[12, 7:8]

RMSE.n.500.SS2[8, ]  <- MSE.99.99[12, 1:2]
RMSE.n.2000.SS2[8, ] <- MSE.99.99[12, 5:6]
RMSE.n.5000.SS2[8, ] <- MSE.99.99[12, 7:8]

##########################################
# Pareto(3) and Pareto(5) distribution #
##########################################

RMSE.n.500.Pareto3 <- RMSE.n.500.fGARCH7 <- RMSE.n.2000.Pareto3 <- RMSE.n.2000.Pareto5 <- RMSE.n.5000.Pareto3 <- RMSE.n.5000.Pareto5 <- matrix(NA, nrow=8, ncol=2)

RMSE.n.500.Pareto3[1, ]  <- MSE.90[14, 1:2]
RMSE.n.2000.Pareto3[1, ] <- MSE.90[14, 5:6]
RMSE.n.5000.Pareto3[1, ] <- MSE.90[14, 7:8]

RMSE.n.500.Pareto3[2, ]  <- MSE.95[14, 1:2]
RMSE.n.2000.Pareto3[2, ] <- MSE.95[14, 5:6]
RMSE.n.5000.Pareto3[2, ] <- MSE.95[14, 7:8]

RMSE.n.500.Pareto3[3, ]  <- MSE.975[14, 1:2]
RMSE.n.2000.Pareto3[3, ] <- MSE.975[14, 5:6]
RMSE.n.5000.Pareto3[3, ] <- MSE.975[14, 7:8]

RMSE.n.500.Pareto3[4, ]  <- MSE.99[14, 1:2]
RMSE.n.2000.Pareto3[4, ] <- MSE.99[14, 5:6]
RMSE.n.5000.Pareto3[4, ] <- MSE.99[14, 7:8]

RMSE.n.500.Pareto3[5, ]  <- MSE.99.5[14, 1:2]
RMSE.n.2000.Pareto3[5, ] <- MSE.99.5[14, 5:6]
RMSE.n.5000.Pareto3[5, ] <- MSE.99.5[14, 7:8]

RMSE.n.500.Pareto3[6, ]  <- MSE.99.9[14, 1:2]
RMSE.n.2000.Pareto3[6, ] <- MSE.99.9[14, 5:6]
RMSE.n.5000.Pareto3[6, ] <- MSE.99.9[14, 7:8]

RMSE.n.500.Pareto3[7, ]  <- MSE.99.95[14, 1:2]
RMSE.n.2000.Pareto3[7, ] <- MSE.99.95[14, 5:6]
RMSE.n.5000.Pareto3[7, ] <- MSE.99.95[14, 7:8]

RMSE.n.500.Pareto3[8, ]  <- MSE.99.99[14, 1:2]
RMSE.n.2000.Pareto3[8, ] <- MSE.99.99[14, 5:6]
RMSE.n.5000.Pareto3[8, ] <- MSE.99.99[14, 7:8]


RMSE.n.500.Pareto5[1, ]  <- MSE.90[15, 1:2]
RMSE.n.2000.Pareto5[1, ] <- MSE.90[15, 5:6]
RMSE.n.5000.Pareto5[1, ] <- MSE.90[15, 7:8]

RMSE.n.500.Pareto5[2, ]  <- MSE.95[15, 1:2]
RMSE.n.2000.Pareto5[2, ] <- MSE.95[15, 5:6]
RMSE.n.5000.Pareto5[2, ] <- MSE.95[15, 7:8]

RMSE.n.500.Pareto5[3, ]  <- MSE.975[15, 1:2]
RMSE.n.2000.Pareto5[3, ] <- MSE.975[15, 5:6]
RMSE.n.5000.Pareto5[3, ] <- MSE.975[15, 7:8]

RMSE.n.500.Pareto5[4, ]  <- MSE.99[15, 1:2]
RMSE.n.2000.Pareto5[4, ] <- MSE.99[15, 5:6]
RMSE.n.5000.Pareto5[4, ] <- MSE.99[15, 7:8]

RMSE.n.500.Pareto5[5, ]  <- MSE.99.5[15, 1:2]
RMSE.n.2000.Pareto5[5, ] <- MSE.99.5[15, 5:6]
RMSE.n.5000.Pareto5[5, ] <- MSE.99.5[15, 7:8]

RMSE.n.500.Pareto5[6, ]  <- MSE.99.9[15, 1:2]
RMSE.n.2000.Pareto5[6, ] <- MSE.99.9[15, 5:6]
RMSE.n.5000.Pareto5[6, ] <- MSE.99.9[15, 7:8]

RMSE.n.500.Pareto5[7, ]  <- MSE.99.95[15, 1:2]
RMSE.n.2000.Pareto5[7, ] <- MSE.99.95[15, 5:6]
RMSE.n.5000.Pareto5[7, ] <- MSE.99.95[15, 7:8]

RMSE.n.500.Pareto5[8, ]  <- MSE.99.99[15, 1:2]
RMSE.n.2000.Pareto5[8, ] <- MSE.99.99[15, 5:6]
RMSE.n.5000.Pareto5[8, ] <- MSE.99.99[15, 7:8]


#######################
# Burr(3) and Burr(5) #
#######################

RMSE.n.500.Burr3 <- RMSE.n.500.Burr5 <- RMSE.n.2000.Burr3 <- RMSE.n.2000.Burr5 <- RMSE.n.5000.Burr3 <- RMSE.n.5000.Burr5 <- matrix(NA, nrow=8, ncol=2)

RMSE.n.500.Burr3[1, ]  <- MSE.90[18, 1:2]
RMSE.n.2000.Burr3[1, ] <- MSE.90[18, 5:6]
RMSE.n.5000.Burr3[1, ] <- MSE.90[18, 7:8]

RMSE.n.500.Burr3[2, ]  <- MSE.95[18, 1:2]
RMSE.n.2000.Burr3[2, ] <- MSE.95[18, 5:6]
RMSE.n.5000.Burr3[2, ] <- MSE.95[18, 7:8]

RMSE.n.500.Burr3[3, ]  <- MSE.975[18, 1:2]
RMSE.n.2000.Burr3[3, ] <- MSE.975[18, 5:6]
RMSE.n.5000.Burr3[3, ] <- MSE.975[18, 7:8]

RMSE.n.500.Burr3[4, ]  <- MSE.99[18, 1:2]
RMSE.n.2000.Burr3[4, ] <- MSE.99[18, 5:6]
RMSE.n.5000.Burr3[4, ] <- MSE.99[18, 7:8]

RMSE.n.500.Burr3[5, ]  <- MSE.99.5[18, 1:2]
RMSE.n.2000.Burr3[5, ] <- MSE.99.5[18, 5:6]
RMSE.n.5000.Burr3[5, ] <- MSE.99.5[18, 7:8]

RMSE.n.500.Burr3[6, ]  <- MSE.99.9[18, 1:2]
RMSE.n.2000.Burr3[6, ] <- MSE.99.9[18, 5:6]
RMSE.n.5000.Burr3[6, ] <- MSE.99.9[18, 7:8]

RMSE.n.500.Burr3[7, ]  <- MSE.99.95[18, 1:2]
RMSE.n.2000.Burr3[7, ] <- MSE.99.95[18, 5:6]
RMSE.n.5000.Burr3[7, ] <- MSE.99.95[18, 7:8]

RMSE.n.500.Burr3[8, ]  <- MSE.99.99[18, 1:2]
RMSE.n.2000.Burr3[8, ] <- MSE.99.99[18, 5:6]
RMSE.n.5000.Burr3[8, ] <- MSE.99.99[18, 7:8]


RMSE.n.500.Burr5[1, ]  <- MSE.90[19, 1:2]
RMSE.n.2000.Burr5[1, ] <- MSE.90[19, 5:6]
RMSE.n.5000.Burr5[1, ] <- MSE.90[19, 7:8]

RMSE.n.500.Burr5[2, ]  <- MSE.95[19, 1:2]
RMSE.n.2000.Burr5[2, ] <- MSE.95[19, 5:6]
RMSE.n.5000.Burr5[2, ] <- MSE.95[19, 7:8]

RMSE.n.500.Burr5[3, ]  <- MSE.975[19, 1:2]
RMSE.n.2000.Burr5[3, ] <- MSE.975[19, 5:6]
RMSE.n.5000.Burr5[3, ] <- MSE.975[19, 7:8]

RMSE.n.500.Burr5[4, ]  <- MSE.99[19, 1:2]
RMSE.n.2000.Burr5[4, ] <- MSE.99[19, 5:6]
RMSE.n.5000.Burr5[4, ] <- MSE.99[19, 7:8]

RMSE.n.500.Burr5[5, ]  <- MSE.99.5[19, 1:2]
RMSE.n.2000.Burr5[5, ] <- MSE.99.5[19, 5:6]
RMSE.n.5000.Burr5[5, ] <- MSE.99.5[19, 7:8]

RMSE.n.500.Burr5[6, ]  <- MSE.99.9[19, 1:2]
RMSE.n.2000.Burr5[6, ] <- MSE.99.9[19, 5:6]
RMSE.n.5000.Burr5[6, ] <- MSE.99.9[19, 7:8]

RMSE.n.500.Burr5[7, ]  <- MSE.99.95[19, 1:2]
RMSE.n.2000.Burr5[7, ] <- MSE.99.95[19, 5:6]
RMSE.n.5000.Burr5[7, ] <- MSE.99.95[19, 7:8]

RMSE.n.500.Burr5[8, ]  <- MSE.99.99[19, 1:2]
RMSE.n.2000.Burr5[8, ] <- MSE.99.99[19, 5:6]
RMSE.n.5000.Burr5[8, ] <- MSE.99.99[19, 7:8]


###################
# Plot Figure A.2 #
###################

par(mfrow=c(3, 2))

alpha     = c(0.9, 0.95, 0.975, 0.99, 0.995, 0.999, 0.9995, 0.9999)
ticks = 1-alpha

# 1. model
options(scipen=999)
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.n.500.SS1[ , 1] / RMSE.n.500.SS1[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.n.2000.SS1[ , 1] / RMSE.n.2000.SS1[ , 2], lty=2)
lines(1-alpha, RMSE.n.5000.SS1[ , 1] / RMSE.n.5000.SS1[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(a1) SS-distribution ("~alpha ~"= 1.0 )"), side=1, line = 4)


# 2. model
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.n.500.SS2[ , 1] / RMSE.n.500.SS2[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.n.2000.SS2[ , 1] / RMSE.n.2000.SS2[ , 2], lty=2)
lines(1-alpha, RMSE.n.5000.SS2 [ , 1] / RMSE.n.5000.SS2[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(a2) SS-distribution ("~alpha ~"= 1.9 )"), side=1, line = 4)


# 3. model
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.n.500.Pareto3[ , 1] / RMSE.n.500.Pareto3[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.n.2000.Pareto3[ , 1] / RMSE.n.2000.Pareto3[ , 2], lty=2)
lines(1-alpha, RMSE.n.5000.Pareto3[ , 1] / RMSE.n.5000.Pareto3[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(b1) Pareto-distribution ("~alpha ~"= 3 )"), side=1, line = 4)


# 4. model
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.n.500.Pareto5[ , 1] / RMSE.n.500.Pareto5[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.n.2000.Pareto5[ , 1] / RMSE.n.2000.Pareto5[ , 2], lty=2)
lines(1-alpha, RMSE.n.5000.Pareto5[ , 1] / RMSE.n.5000.Pareto5[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(b2) Pareto-distribution ("~alpha ~"= 5 )"), side=1, line = 4)


# 5. model
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.n.500.Burr3[ , 1] / RMSE.n.500.Burr3[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.n.2000.Burr3[ , 1] / RMSE.n.2000.Burr3[ , 2], lty=2)
lines(1-alpha, RMSE.n.5000.Burr3[ , 1] / RMSE.n.5000.Burr3[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(c1) Burr-distribution ("~alpha ~"= 3 )"), side=1, line = 4)


# 6. model
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.n.500.Burr5[ , 1] / RMSE.n.500.Burr5[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.n.2000.Burr5[ , 1] / RMSE.n.2000.Burr5[ , 2], lty=2)
lines(1-alpha, RMSE.n.5000.Burr5[ , 1] / RMSE.n.5000.Burr5[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(c2) Burr-distribution ("~alpha ~"= 5 )"), side=1, line = 4)



# ===================================================
# Figure A.3: RMSE ratios for VaR estimation for (M8)-(M9)
# ===================================================

##################################
# GARCH3 and GARCH5 distribution #
##################################

RMSE.n.500.GARCH3 <- RMSE.n.500.GARCH5 <- RMSE.n.2000.GARCH3 <- RMSE.n.2000.GARCH5 <- RMSE.n.5000.GARCH3 <- RMSE.n.5000.GARCH5 <- matrix(NA, nrow=8, ncol=2)

RMSE.n.500.GARCH3[1, ]  <- MSE.90[26, 1:2]
RMSE.n.2000.GARCH3[1, ] <- MSE.90[26, 5:6]
RMSE.n.5000.GARCH3[1, ] <- MSE.90[26, 7:8]
RMSE.n.500.GARCH3[2, ]  <- MSE.95[26, 1:2]
RMSE.n.2000.GARCH3[2, ] <- MSE.95[26, 5:6]
RMSE.n.5000.GARCH3[2, ] <- MSE.95[26, 7:8]
RMSE.n.500.GARCH3[3, ]  <- MSE.975[26, 1:2]
RMSE.n.2000.GARCH3[3, ] <- MSE.975[26, 5:6]
RMSE.n.5000.GARCH3[3, ] <- MSE.975[26, 7:8]
RMSE.n.500.GARCH3[4, ]  <- MSE.99[26, 1:2]
RMSE.n.2000.GARCH3[4, ] <- MSE.99[26, 5:6]
RMSE.n.5000.GARCH3[4, ] <- MSE.99[26, 7:8]
RMSE.n.500.GARCH3[5, ]  <- MSE.99.5[26, 1:2]
RMSE.n.2000.GARCH3[5, ] <- MSE.99.5[26, 5:6]
RMSE.n.5000.GARCH3[5, ] <- MSE.99.5[26, 7:8]
RMSE.n.500.GARCH3[6, ]  <- MSE.99.9[26, 1:2]
RMSE.n.2000.GARCH3[6, ] <- MSE.99.9[26, 5:6]
RMSE.n.5000.GARCH3[6, ] <- MSE.99.9[26, 7:8]
RMSE.n.500.GARCH3[7, ]  <- MSE.99.95[26, 1:2]
RMSE.n.2000.GARCH3[7, ] <- MSE.99.95[26, 5:6]
RMSE.n.5000.GARCH3[7, ] <- MSE.99.95[26, 7:8]
RMSE.n.500.GARCH3[8, ]  <- MSE.99.99[26, 1:2]
RMSE.n.2000.GARCH3[8, ] <- MSE.99.99[26, 5:6]
RMSE.n.5000.GARCH3[8, ] <- MSE.99.99[26, 7:8]


RMSE.n.500.GARCH5[1, ]  <- MSE.90[28, 1:2]
RMSE.n.2000.GARCH5[1, ] <- MSE.90[28, 5:6]
RMSE.n.5000.GARCH5[1, ] <- MSE.90[28, 7:8]

RMSE.n.500.GARCH5[2, ]  <- MSE.95[28, 1:2]
RMSE.n.2000.GARCH5[2, ] <- MSE.95[28, 5:6]
RMSE.n.5000.GARCH5[2, ] <- MSE.95[28, 7:8]

RMSE.n.500.GARCH5[3, ]  <- MSE.975[28, 1:2]
RMSE.n.2000.GARCH5[3, ] <- MSE.975[28, 5:6]
RMSE.n.5000.GARCH5[3, ] <- MSE.975[28, 7:8]

RMSE.n.500.GARCH5[4, ]  <- MSE.99[28, 1:2]
RMSE.n.2000.GARCH5[4, ] <- MSE.99[28, 5:6]
RMSE.n.5000.GARCH5[4, ] <- MSE.99[28, 7:8]

RMSE.n.500.GARCH5[5, ]  <- MSE.99.5[28, 1:2]
RMSE.n.2000.GARCH5[5, ] <- MSE.99.5[28, 5:6]
RMSE.n.5000.GARCH5[5, ] <- MSE.99.5[28, 7:8]

RMSE.n.500.GARCH5[6, ]  <- MSE.99.9[28, 1:2]
RMSE.n.2000.GARCH5[6, ] <- MSE.99.9[28, 5:6]
RMSE.n.5000.GARCH5[6, ] <- MSE.99.9[28, 7:8]

RMSE.n.500.GARCH5[7, ]  <- MSE.99.95[28, 1:2]
RMSE.n.2000.GARCH5[7, ] <- MSE.99.95[28, 5:6]
RMSE.n.5000.GARCH5[7, ] <- MSE.99.95[28, 7:8]

RMSE.n.500.GARCH5[8, ]  <- MSE.99.99[28, 1:2]
RMSE.n.2000.GARCH5[8, ] <- MSE.99.99[28, 5:6]
RMSE.n.5000.GARCH5[8, ] <- MSE.99.99[28, 7:8]

####################################
# fGARCH5 and fGARCH7 distribution #
####################################

RMSE.n.500.fGARCH5 <- RMSE.n.500.fGARCH7 <- RMSE.n.2000.fGARCH5 <- RMSE.n.2000.fGARCH7 <- RMSE.n.5000.fGARCH5 <- RMSE.n.5000.fGARCH7 <- matrix(NA, nrow=8, ncol=2)

RMSE.n.500.fGARCH5[1, ]  <- MSE.90[29, 1:2]
RMSE.n.2000.fGARCH5[1, ] <- MSE.90[29, 5:6]
RMSE.n.5000.fGARCH5[1, ] <- MSE.90[29, 7:8]

RMSE.n.500.fGARCH5[2, ]  <- MSE.95[29, 1:2]
RMSE.n.2000.fGARCH5[2, ] <- MSE.95[29, 5:6]
RMSE.n.5000.fGARCH5[2, ] <- MSE.95[29, 7:8]

RMSE.n.500.fGARCH5[3, ]  <- MSE.975[29, 1:2]
RMSE.n.2000.fGARCH5[3, ] <- MSE.975[29, 5:6]
RMSE.n.5000.fGARCH5[3, ] <- MSE.975[29, 7:8]

RMSE.n.500.fGARCH5[4, ]  <- MSE.99[29, 1:2]
RMSE.n.2000.fGARCH5[4, ] <- MSE.99[29, 5:6]
RMSE.n.5000.fGARCH5[4, ] <- MSE.99[29, 7:8]

RMSE.n.500.fGARCH5[5, ]  <- MSE.99.5[29, 1:2]
RMSE.n.2000.fGARCH5[5, ] <- MSE.99.5[29, 5:6]
RMSE.n.5000.fGARCH5[5, ] <- MSE.99.5[29, 7:8]

RMSE.n.500.fGARCH5[6, ]  <- MSE.99.9[29, 1:2]
RMSE.n.2000.fGARCH5[6, ] <- MSE.99.9[29, 5:6]
RMSE.n.5000.fGARCH5[6, ] <- MSE.99.9[29, 7:8]

RMSE.n.500.fGARCH5[7, ]  <- MSE.99.95[29, 1:2]
RMSE.n.2000.fGARCH5[7, ] <- MSE.99.95[29, 5:6]
RMSE.n.5000.fGARCH5[7, ] <- MSE.99.95[29, 7:8]

RMSE.n.500.fGARCH5[8, ]  <- MSE.99.99[29, 1:2]
RMSE.n.2000.fGARCH5[8, ] <- MSE.99.99[29, 5:6]
RMSE.n.5000.fGARCH5[8, ] <- MSE.99.99[29, 7:8]


RMSE.n.500.fGARCH7[1, ]  <- MSE.90[30, 1:2]
RMSE.n.2000.fGARCH7[1, ] <- MSE.90[30, 5:6]
RMSE.n.5000.fGARCH7[1, ] <- MSE.90[30, 7:8]

RMSE.n.500.fGARCH7[2, ]  <- MSE.95[30, 1:2]
RMSE.n.2000.fGARCH7[2, ] <- MSE.95[30, 5:6]
RMSE.n.5000.fGARCH7[2, ] <- MSE.95[30, 7:8]

RMSE.n.500.fGARCH7[3, ]  <- MSE.975[30, 1:2]
RMSE.n.2000.fGARCH7[3, ] <- MSE.975[30, 5:6]
RMSE.n.5000.fGARCH7[3, ] <- MSE.975[30, 7:8]

RMSE.n.500.fGARCH7[4, ]  <- MSE.99[30, 1:2]
RMSE.n.2000.fGARCH7[4, ] <- MSE.99[30, 5:6]
RMSE.n.5000.fGARCH7[4, ] <- MSE.99[30, 7:8]

RMSE.n.500.fGARCH7[5, ]  <- MSE.99.5[30, 1:2]
RMSE.n.2000.fGARCH7[5, ] <- MSE.99.5[30, 5:6]
RMSE.n.5000.fGARCH7[5, ] <- MSE.99.5[30, 7:8]

RMSE.n.500.fGARCH7[6, ]  <- MSE.99.9[30, 1:2]
RMSE.n.2000.fGARCH7[6, ] <- MSE.99.9[30, 5:6]
RMSE.n.5000.fGARCH7[6, ] <- MSE.99.9[30, 7:8]

RMSE.n.500.fGARCH7[7, ]  <- MSE.99.95[30, 1:2]
RMSE.n.2000.fGARCH7[7, ] <- MSE.99.95[30, 5:6]
RMSE.n.5000.fGARCH7[7, ] <- MSE.99.95[30, 7:8]

RMSE.n.500.fGARCH7[8, ]  <- MSE.99.99[30, 1:2]
RMSE.n.2000.fGARCH7[8, ] <- MSE.99.99[30, 5:6]
RMSE.n.5000.fGARCH7[8, ] <- MSE.99.99[30, 7:8]





###################
# Plot Figure A.3 #
###################

par(mfrow=c(2, 2))

alpha     = c(0.9, 0.95, 0.975, 0.99, 0.995, 0.999, 0.9995, 0.9999)
ticks = 1-alpha

# 1. model
options(scipen=999)
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.n.500.GARCH3[ , 1] / RMSE.n.500.GARCH3[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.n.2000.GARCH3[ , 1] / RMSE.n.2000.GARCH3[ , 2], lty=2)
lines(1-alpha, RMSE.n.5000.GARCH3[ , 1] / RMSE.n.5000.GARCH3[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(a1) GARCH ("~alpha ~"= 2.98 )"), side=1, line = 4)


# 2. model
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.n.500.GARCH5[ , 1] / RMSE.n.500.GARCH5[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.n.2000.GARCH5[ , 1] / RMSE.n.2000.GARCH5[ , 2], lty=2)
lines(1-alpha, RMSE.n.5000.GARCH5 [ , 1] / RMSE.n.5000.GARCH5[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(a2) GARCH ("~alpha ~"= 4.99 )"), side=1, line = 4)


# 3. model
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.n.500.fGARCH5[ , 1] / RMSE.n.500.fGARCH5[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.n.2000.fGARCH5[ , 1] / RMSE.n.2000.fGARCH5[ , 2], lty=2)
lines(1-alpha, RMSE.n.5000.fGARCH5[ , 1] / RMSE.n.5000.fGARCH5[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(b1) Filtered GARCH ("~alpha ~"= 5 )"), side=1, line = 4)


# 4. model
par(mar=c(5.1,4,2,2))
plot(1-alpha, RMSE.n.500.fGARCH7[ , 1] / RMSE.n.500.fGARCH7[ , 2], log="xy", lty=1, xlim=c(0.0001, 0.10), ylim=c(0.5, 50), yaxs="i",
     xlab = "", ylab = "RMSE ratio", las=1, xaxt="n", type="l")
axis(side = 1, at = ticks, labels=rev(c(0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025, 0.05, 0.1)))
lines(1-alpha, RMSE.n.2000.fGARCH7[ , 1] / RMSE.n.2000.fGARCH7[ , 2], lty=2)
lines(1-alpha, RMSE.n.5000.fGARCH7[ , 1] / RMSE.n.5000.fGARCH7[ , 2], lty=3)
abline(h=1)
legend("topright", legend=c("n=500", "n=2000", "n=5000"), lty=c(1, 2, 3))
mtext(expression(italic(1-p)), side=1, line=2.5)
mtext(expression("(b2) Filtered GARCH ("~alpha ~"= 7 )"), side=1, line = 4)




