# ===================================================
# ===================================================
# Functions to replicate simulation results in the paper
# ===================================================
# ===================================================



# ===========================================================
# Function to calculate RMSE of tail estimates, viz. tail index, VaR, ES
# ===========================================================

# Inputs: 
# d: specifies the DGP
# n: length of samples for estimation
# reps: number of Monte Carlo repetitions
# k.max: maximal number of k entertained
# estimator: either "Hill" for Hill (1975) estimator or "GI" for Gabaix & Ibragimov (2011) estimator


RMSE <- function(d, n, reps, k.max, estimator){   # estimator ="Hill" for Hill estimator or "GI" for Gabaix & Ibragimov (2011)
  probs         <- c(0.90, 0.95, 0.975, 0.99, 0.995, 0.999, 0.9995, 0.9999)
  quantile.true <- ES.true <- numeric(length(probs))
  gamma.est     <- k.est <- matrix(NA, nrow=reps, ncol=2)     # save all gamma estimates to produce Figure 2 in paper
  VaR.MAX.est <- ES.MAX.est <- matrix(NA, nrow=reps, ncol=length(probs))
  VaR.SCR.est <- ES.SCR.est <- matrix(NA, nrow=reps, ncol=length(probs))
  
  gamma.fix <- k.fix <- rep.int(NA, reps)
  
  MSE     <- Bias   <- rep.int(0, 2)
  MSE.VaR <- MSE.ES <- matrix(0, nrow=length(probs), ncol=2)
  for(i in 1:reps){
    
    # t distribution
    if(d==1){
      X <- rt(n, df=1)
      gamma.true <- 1 / 1
      quantile.true <- qt(p=probs, df=1)
      k.avg <- 26* 1*(n==500) + 39 * 1*(n==1000) + 57 * 1*(n==2000) + 97 * 1*(n==5000) # save average k reported in Table 3 for later use
    }
    if(d==2){
      X <- rt(n, df=3)
      gamma.true <- 1 / 3
      quantile.true <- qt(p=probs, df=3)
      for(j in 1:length(probs)){
        ES.true[j] <- 1/(1-probs[j]) * integrate(qt, lower=probs[j], upper=1, df=3)$value
      }
      k.avg <- 28* 1*(n==500) + 43 * 1*(n==1000) + 63 * 1*(n==2000) + 108 * 1*(n==5000)
    }
    if(d==3){
      X <- rt(n, df=5)
      gamma.true <- 1 / 5
      quantile.true <- qt(p=probs, df=5)
      for(j in 1:length(probs)){
        ES.true[j] <- 1/(1-probs[j]) * integrate(qt, lower=probs[j], upper=1, df=5)$value
      }
      k.avg <- 29* 1*(n==500) + 44 * 1*(n==1000) + 65 * 1*(n==2000) + 114 * 1*(n==5000)
    }
    if(d==4){
      X <- rt(n, df=7)
      gamma.true <- 1 / 7
      quantile.true <- qt(p=probs, df=7)
      for(j in 1:length(probs)){
        ES.true[j] <- 1/(1-probs[j]) * integrate(qt, lower=probs[j], upper=1, df=7)$value
      }
      k.avg <- 29* 1*(n==500) + 44 * 1*(n==1000) + 66 * 1*(n==2000) + 116 * 1*(n==5000)
    }
     
     # Frechet distributions
     if(d==5){
        X <- rfrechet(n, shape=1)
        gamma.true <- 1 / 1
        quantile.true <- qfrechet(p=probs, shape=1)
        k.avg <- 26* 1*(n==500) + 39 * 1*(n==1000) + 57 * 1*(n==2000) + 96 * 1*(n==5000)
     }
     if(d==6){
        X <- rfrechet(n, shape=3)
        gamma.true <- 1 / 3
        quantile.true <- qfrechet(p=probs, shape=3)
        for(j in 1:length(probs)){
          ES.true[j] <- 1/(1-probs[j]) * integrate(qfrechet, lower=probs[j], upper=1, shape=3)$value
        }
        k.avg <- 29* 1*(n==500) + 43 * 1*(n==1000) + 63 * 1*(n==2000) + 108 * 1*(n==5000)
     }
     if(d==7){
        X <- rfrechet(n, shape=5)
        gamma.true <- 1 / 5
        quantile.true <- qfrechet(p=probs, shape=5)
        for(j in 1:length(probs)){
          ES.true[j] <- 1/(1-probs[j]) * integrate(qfrechet, lower=probs[j], upper=1, shape=5)$value
        }
        k.avg <- 29* 1*(n==500) + 44 * 1*(n==1000) + 64 * 1*(n==2000) + 108 * 1*(n==5000)
     }
     if(d==8){
        X <- rfrechet(n, shape=7)
        gamma.true <- 1 / 7
        quantile.true <- qfrechet(p=probs, shape=7)
        for(j in 1:length(probs)){
          ES.true[j] <- 1/(1-probs[j]) * integrate(qfrechet, lower=probs[j], upper=1, shape=7)$value
        }
        k.avg <- 30* 1*(n==500) + 44 * 1*(n==1000) + 65 * 1*(n==2000) + 109 * 1*(n==5000)
     }
     
     
     # Stable distributions
     if(d==9){
        alpha         <- 0.5
        X             <- rstable(n, alpha, beta=0)
        gamma.true    <- 1 / alpha
        quantile.true <- qstable(p=probs, alpha, beta=0)
        k.avg <- 20* 1*(n==500) + 28 * 1*(n==1000) + 38 * 1*(n==2000) + 59 * 1*(n==5000)
     }
     if(d==10){
        alpha <- 1.0
        X <- rstable(n, alpha, beta=0)
        gamma.true <- 1 / alpha
        quantile.true <- qstable(p=probs, alpha, beta=0)
        k.avg <- 26* 1*(n==500) + 39 * 1*(n==1000) + 57 * 1*(n==2000) + 97 * 1*(n==5000)
     }
     if(d==11){
        alpha         <- 1.5
        X             <- rstable(n, alpha, beta=0)
        gamma.true    <- 1 / alpha
        quantile.true <- qstable(p=probs, alpha, beta=0)
        k.avg <- 28* 1*(n==500) + 42 * 1*(n==1000) + 61 * 1*(n==2000) + 103 * 1*(n==5000)
     }
     if(d==12){
        alpha <- 1.9
        X     <- rstable(n, alpha, beta=0)
        gamma.true    <- 1 / alpha
        quantile.true <- qstable(p=probs, alpha, beta=0)
        k.avg <- 27* 1*(n==500) + 41 * 1*(n==1000) + 60 * 1*(n==2000) + 106 * 1*(n==5000)
     }
     
     
     # Pareto distribution
     if(d==13){
        X <- rlgamma(n, shapelog=1, ratelog=1)
        gamma.true <- 1 / 1
        quantile.true <- qlgamma(p=probs, shapelog=1, ratelog=1)
        # for(j in 1:length(probs)){
        #   ES.true[j] <- 1/(1-probs[j]) * integrate(qlgamma, lower=probs[j], upper=1, shapelog=1, ratelog=1)$value
        # }
        k.avg <- 26* 1*(n==500) + 39 * 1*(n==1000) + 57 * 1*(n==2000) + 97 * 1*(n==5000)
     }
     if(d==14){
        X <- rlgamma(n, shapelog=1, ratelog=3)
        gamma.true <- 1 / 3
        quantile.true <- qlgamma(p=probs, shapelog=1, ratelog=3)
        for(j in 1:length(probs)){
          ES.true[j] <- 1/(1-probs[j]) * integrate(qlgamma, lower=probs[j], upper=1, shapelog=1, ratelog=3)$value
        }
        k.avg <- 29* 1*(n==500) + 43 * 1*(n==1000) + 63 * 1*(n==2000) + 107 * 1*(n==5000)
     }
     if(d==15){
        X <- rlgamma(n, shapelog=1, ratelog=5)
        gamma.true <- 1 / 5
        quantile.true <- qlgamma(p=probs, shapelog=1, ratelog=5)
        for(j in 1:length(probs)){
          ES.true[j] <- 1/(1-probs[j]) * integrate(qlgamma, lower=probs[j], upper=1, shapelog=1, ratelog=5)$value
        }
        k.avg <- 29* 1*(n==500) + 44 * 1*(n==1000) + 64 * 1*(n==2000) + 108 * 1*(n==5000)
     }
     if(d==16){
        X <- rlgamma(n, shapelog=1, ratelog=7)
        gamma.true <- 1 / 7
        quantile.true <- qlgamma(p=probs, shapelog=1, ratelog=7)
        for(j in 1:length(probs)){
          ES.true[j] <- 1/(1-probs[j]) * integrate(qlgamma, lower=probs[j], upper=1, shapelog=1, ratelog=7)$value
        }
        k.avg <- 29* 1*(n==500) + 44 * 1*(n==1000) + 65 * 1*(n==2000) + 109 * 1*(n==5000)
     }
    
    # Burr distribution
    if(d==17){
      lambda <- 1/2
      tau    <- 2
      X <- rburr(n, shape1 = lambda, shape2 = tau)
      gamma.true <- 1 / (lambda * tau)
      quantile.true <- qburr(p=probs, shape1 = lambda, shape2 = tau)
      k.avg <- 26* 1*(n==500) + 39 * 1*(n==1000) + 57 * 1*(n==2000) + 97 * 1*(n==5000)
    }
    if(d==18){
      lambda <- 3/2
      tau    <- 2
      X <- rburr(n, shape1 = lambda, shape2 = tau)
      gamma.true <- 1 / (lambda * tau)
      quantile.true <- qburr(p=probs, shape1 = lambda, shape2 = tau)
      for(j in 1:length(probs)){
        ES.true[j] <- 1/(1-probs[j]) * integrate(qburr, lower=probs[j], upper=1, shape1 = lambda, shape2 = tau)$value
      }
      k.avg <- 29* 1*(n==500) + 43 * 1*(n==1000) + 63 * 1*(n==2000) + 107 * 1*(n==5000)
    }
    if(d==19){
      lambda <- 5/2
      tau    <- 2
      X <- rburr(n, shape1 = lambda, shape2 = tau)
      gamma.true <- 1 / (lambda * tau)
      quantile.true <- qburr(p=probs, shape1 = lambda, shape2 = tau)
      for(j in 1:length(probs)){
        ES.true[j] <- 1/(1-probs[j]) * integrate(qburr, lower=probs[j], upper=1, shape1 = lambda, shape2 = tau)$value
      }
      k.avg <- 29* 1*(n==500) + 44 * 1*(n==1000) + 65 * 1*(n==2000) + 110 * 1*(n==5000)
    }
    if(d==20){
      lambda <- 7/2
      tau    <- 2
      X <- rburr(n, shape1 = lambda, shape2 = tau)
      gamma.true <- 1 / (lambda * tau)
      quantile.true <- qburr(p=probs, shape1 = lambda, shape2 = tau)
      for(j in 1:length(probs)){
        ES.true[j] <- 1/(1-probs[j]) * integrate(qburr, lower=probs[j], upper=1, shape1 = lambda, shape2 = tau)$value
      }
      k.avg <- 29* 1*(n==500) + 44 * 1*(n==1000) + 66 * 1*(n==2000) + 113 * 1*(n==5000)
    }

    
    # Stationary ARCH(1) distributions (good for 2, 3, 4, 5)
    if(d==21){   
      spec.5 = garchSpec(model = list(omega=10e-06, alpha = 0.9, beta = 0.0), cond.dist = "norm")
      X <- as.numeric(garchSim(spec.5, n))
      gamma.true <- 1 / 2.304337   # see Support.R script for the computation of this gamma, quantile.true and ES.true
      quantile.true <- c(0.006771530, 0.009862934, 0.01384391, 0.021097020, 0.028739026, 0.058214747, 0.078715781, 0.159177528)
      ES.true       <- c(0.013201723, 0.018319347, 0.02510214, 0.03770020, 0.05110769, 0.1032608, 0.13993316, 0.2873509)
      k.avg <- 28* 1*(n==500) + 42 * 1*(n==1000) + 63 * 1*(n==2000) + 106 * 1*(n==5000)
    }
    if(d==22){
      spec.6 = garchSpec(model = list(omega=10e-06, alpha = 0.8, beta = 0.0), cond.dist = "norm")
      X <- as.numeric(garchSim(spec.6, n))
      gamma.true <- 1 / 2.684231   # see Support.R script for the computation of this gamma, quantile.true and ES.true
      quantile.true <- c(0.006305929, 0.008918529, 0.01208694 , 0.017558005, 0.023008283, 0.042419966, 0.055017351, 0.100321121)
      ES.true       <- c(0.011267230, 0.015104022, 0.01994557, 0.02845729, 0.0370608, 0.06810859, 0.08860344, 0.1663294)
      k.avg <- 29* 1*(n==500) + 43 * 1*(n==1000) + 63 * 1*(n==2000) + 107 * 1*(n==5000)
    }
    if(d==23){
      spec.7 = garchSpec(model = list(omega=10e-06, alpha = 0.7, beta = 0.0), cond.dist = "norm")
      X <- as.numeric(garchSim(spec.7, n))
      gamma.true <- 1 / 3.172043   # see Support.R script for the computation of this gamma, quantile.true and ES.true
      quantile.true <- c( 0.005903651, 0.008148554, 0.01070296, 0.014885979, 0.018844431, 0.031880312, 0.039786826, 0.066060757)
      ES.true       <- c(0.00983688, 0.012791327, 0.01633902, 0.02225956, 0.02794896, 0.04705726, 0.05893912, 0.10173960)
      k.avg <- 29* 1*(n==500) + 43 * 1*(n==1000) + 64 * 1*(n==2000) + 109 * 1*(n==5000)
    }
    if(d==24){
      spec.8 = garchSpec(model = list(omega=10e-06, alpha = 0.6, beta = 0.0), cond.dist = "norm")
      X <- as.numeric(garchSim(spec.8, n))
      gamma.true <- 1 / 3.821705   # see Support.R script for the computation of this gamma, quantile.true and ES.true
      quantile.true <- c(0.005551502, 0.007511994, 0.009605222, 0.012838583, 0.015760996, 0.024695101, 0.029769425, 0.045462247)
      ES.true       <- c(0.008747858, 0.011079336, 0.013738625, 0.01795934, 0.02182667, 0.03395440, 0.04108972, 0.06573527)
      k.avg <- 29* 1*(n==500) + 44 * 1*(n==1000) + 64 * 1*(n==2000) + 111 * 1*(n==5000)
    }
     
     
     # Stationary GARCH(1,1) distributions (good for 2,3 not so for 4,5)
     if(d==25){   
        spec.4 = garchSpec(model = list(omega=10e-06, alpha = 0.4, beta = 0.599), cond.dist = "norm")
        X <- as.numeric(garchSim(spec.4, n))
        gamma.true <- 1 / 2.02789   # see Support.R script for the computation of this gamma, quantile.true and ES.true
        quantile.true <- c(0.01752776, 0.02662994, 0.03881053, 0.06232379, 0.08854282, 0.20019653, 0.28822940, 0.66847278)
        ES.true       <- c(0.03823763, 0.05511543, 0.07852130, 0.12437677, 0.17558283, 0.3873380, 0.5394330, 1.0822867)
        k.avg <- 28* 1*(n==500) + 42 * 1*(n==1000) + 63 * 1*(n==2000) + 107 * 1*(n==5000)
     }
     if(d==26){
        spec.3 = garchSpec(model = list(omega=10e-06, alpha = 0.4, beta = 0.53), cond.dist = "norm")
        X <- as.numeric(garchSim(spec.3, n))
        gamma.true <- 1 / 2.975246   # see Support.R script for the computation of this gamma, quantile.true and ES.true
        quantile.true <- c(0.01164403, 0.01635366, 0.02178412, 0.03074547, 0.03939498, 0.06880437, 0.08719827, 0.15142556)
        ES.true       <- c(0.02004874, 0.02640923, 0.03413813, 0.04725683, 0.06009457, 0.10415267, 0.13186300, 0.2292796)
        k.avg <- 29* 1*(n==500) + 43 * 1*(n==1000) + 64 * 1*(n==2000) + 110 * 1*(n==5000)
     }
     if(d==27){
        spec.2 = garchSpec(model = list(omega=10e-06, alpha = 0.4, beta = 0.43), cond.dist = "norm")
        X <- as.numeric(garchSim(spec.2, n))
        gamma.true <- 1 / 3.956559   # see Support.R script for the computation of this gamma, quantile.true and ES.true
        quantile.true <- c(0.008596429, 0.011647878, 0.01486202, 0.019702447, 0.024016111, 0.037081450, 0.044438669, 0.067075898)
        ES.true       <- c(0.013481377, 0.01701853, 0.02098922, 0.02720061, 0.03284451, 0.05033894, 0.06050238, 0.09500196)
        k.avg <- 29* 1*(n==500) + 44 * 1*(n==1000) + 65 * 1*(n==2000) + 111 * 1*(n==5000)
     }
     if(d==28){
        spec.1 = garchSpec(model = list(omega=10e-06, alpha = 0.4, beta = 0.27), cond.dist = "norm")
        X <- as.numeric(garchSim(spec.1, n))
        gamma.true <- 1 / 4.992628   # see Support.R script for the computation of this gamma, quantile.true and ES.true
        quantile.true <- c(0.006521376, 0.008663325, 0.01078669, 0.013781334, 0.016310875, 0.023438987, 0.027186662, 0.037865144)
        ES.true       <- c(0.009738162, 0.011998737, 0.01439876, 0.01794915, 0.02102191, 0.02993433, 0.03484307, 0.05107283)
        k.avg <- 29* 1*(n==500) + 44 * 1*(n==1000) + 66 * 1*(n==2000) + 113 * 1*(n==5000)
     }
    
    # Filtered GARCH(1,1)
    if(d==29){
      spec.3 = garchSpec(model = list(omega=10e-06, alpha = 0.4, beta = 0.53, shape = 5), cond.dist = "std")
      X <- as.numeric( garchSim(spec.3, n) )
      fit.GARCH <- garchFit(~ garch(1,1), data = X, trace = FALSE)
      X         <- X / fit.GARCH@sigma.t
      gamma.true <- 1 / 5
      quantile.true <- qt(p=probs, df=5) * sqrt((5-2)/5)
      for(j in 1:length(probs)){
        ES.true[j] <- sqrt((5-2)/5) * 1/(1-probs[j]) * integrate(qt, lower=probs[j], upper=1, df=5)$value
      }
      k.avg <- 29* 1*(n==500) + 44 * 1*(n==1000) + 66 * 1*(n==2000) + 114 * 1*(n==5000)
    }
    if(d==30){
      spec.3 = garchSpec(model = list(omega=10e-06, alpha = 0.4, beta = 0.53, shape = 7), cond.dist = "std")
      X <- as.numeric( garchSim(spec.3, n) )
      fit.GARCH <- garchFit(~ garch(1,1), data = X, trace = FALSE)
      X         <- X / fit.GARCH@sigma.t
      gamma.true <- 1 / 7
      quantile.true <- qt(p=probs, df=7) * sqrt((7-2)/7)
      for(j in 1:length(probs)){
        ES.true[j] <- sqrt((7-2)/7) * 1/(1-probs[j]) * integrate(qt, lower=probs[j], upper=1, df=7)$value
      }
      k.avg <- 29* 1*(n==500) + 44 * 1*(n==1000) + 66 * 1*(n==2000) + 117 * 1*(n==5000)
    }
  
  

    Res.est <- EVI.k(X, k.max, k.avg, estimator)
    
    gamma.est[i,] <- Res.est$res.stoch[1, ]      # (reps x 2)-vector of EVI estimates with k^{Q} and k^{QCRPS}
    VaR.est       <- Res.est$res.stoch[2:9, ]    # (8x2)-vector of VaR estimates at levels "probs" with k^{Q} and k^{QCRPS}
    ES.est        <- Res.est$res.stoch[10:17, ]  # (8x2)-vector of ES  estimates at levels "probs" with k^{Q} and k^{QCRPS}
    k.est[i,]     <- Res.est$res.stoch[18, ]     # (reps x 2)-vector of k^{Q} and k^{QCRPS}
    
    gamma.fix[i]  <- Res.est$res.fix[1]          # contains EVI estimates with fixed k
    k.fix[i]      <- Res.est$res.fix[2]          # contains value of that fixed k
    
    VaR.MAX.est[i, ] <- VaR.est[ , 1]
    VaR.SCR.est[i, ] <- VaR.est[ , 2]
    ES.MAX.est[i, ]  <- ES.est[ , 1]
    ES.SCR.est[i, ]  <- ES.est[ , 2]
    
    MSE     <- MSE     + (gamma.est[i,] - gamma.true)^2
    Bias    <- Bias    + (gamma.est[i,] - gamma.true)
    MSE.VaR <- MSE.VaR + (VaR.est - quantile.true)^2
    MSE.ES  <- MSE.ES  + (ES.est  - ES.true)^2
  }
  MSE     <- sqrt(MSE / reps)         # RMSE of gamma estimates
  Bias    <- Bias / reps              # Bias of gamma estimates
  MSE.VaR <- sqrt(MSE.VaR / reps)     # RMSE of quantile estimates
  MSE.ES  <- sqrt(MSE.ES / reps)      # RMSE of quantile estimates
  k.mean  <- colMeans(k.est)          # average number of k^{Q} and k^{QCRPS}
  SD.k    <- apply(k.est, 2, sd)      # standard deviation of k^{Q} and k^{QCRPS}
  
  return( list( RMSE.sim = MSE, Bias.sim = Bias, gamma.all = gamma.est, gamma = gamma.true,
                k.sim = k.mean, SD.k.sim = SD.k, k.all = k.est,
                gamma.fix = gamma.fix, k.fix = k.fix,
                MSE.VaR.sim = MSE.VaR, VaR.MAX.all = VaR.MAX.est, VaR.SCR.all = VaR.SCR.est,
                MSE.ES.sim = MSE.ES,   ES.MAX.all = ES.MAX.est, ES.SCR.all = ES.SCR.est
                ) )
}



# ===========================================================
# Function to calculate tail estimates, viz. tail index, VaR, ES
# ===========================================================

# Inputs: 
# X: data
# k.max: maximal number of k entertained
# k.fix: average k chosen by k^{\QCRPS}
# estimator: either "Hill" for Hill (1975) estimator or "GI" for Gabaix & Ibragimov (2011) estimator


EVI.k <- function(X, k.max, k.fix, estimator){  
  # 1. Compute optimal k* using [Dea16+, p. 16] (Tail Index Estimation: Quantile Driven Threshold Selection)
  probs  <- c(0.90, 0.95, 0.975, 0.99, 0.995, 0.999, 0.9995, 0.9999)
  n      <- length(X)
  X.sort <- sort.int(X, method = "quick", decreasing = TRUE)   # focus on the right tail
  
  
  EVI       <- rep.int(Inf, times=k.max)
  sup.MAX   <- rep.int(Inf, times=k.max)
  QCRPS     <- rep.int(0,   times=k.max)

    for(k in 1 : k.max){
      if(estimator=="Hill"){     # Hill (1975, AoS) estimator
        EVI[k]         <- 1/k * sum(log( pmax(1, (X.sort[1 : (k+1)]) / (X.sort[(k+1)]) , 1)))
      }
      if(estimator=="GI"){     # Rank-1/2 estimator due to Gabaix & Ibragimov (2011, JBES)
        y.t   <- log(1:(k+1) - 0.5)    # Rank - 1/2 
        X.t   <- log(X.sort[1 : (k+1)])
        A.n.g <- sum( (X.t - mean(X.t)) * (y.t - mean(y.t)) )
        B.n   <- sum( (X.t - mean(X.t))^2 )
        EVI[k] <- - A.n.g / B.n     # tail index estimate
        EVI[k] <- 1 / EVI[k]         # extreme value index estimate
      }
      
      
      Q.k          <- (k/(1 : k.max))^( EVI[k] ) * (X.sort[(k+1)])  # Weissman estimate
      deviation    <- X.sort[2 : (k.max+1)] - Q.k                 # deviation of Weissman estimates from empirical quantiles
      
      # 1. Max-absolute deviation
      sup.MAX[k]   <- max( abs(deviation))

      # 2. Scoring rule-based method
      for(i in 1:k.max){
        QCRPS[k] <- QCRPS[k] + sum( (1*(X <= Q.k[i]) - (1 - i/n)) * (Q.k[i] - X) )
      }
    }

  
  # 1. Max-absolute deviation
  k.MAX       <- which(sup.MAX == min(sup.MAX))
  Q.k.MAX     <- (k.MAX / (n*(1-probs)))^(EVI[k.MAX]) * X.sort[(k.MAX+1)]
  ES.k.MAX    <- Q.k.MAX / (1 - min(0.9, EVI[k.MAX]))
  
  # 2. Scoring rule-based
  k.SCR       <- which(QCRPS == min(QCRPS))
  Q.k.SCR     <- (k.SCR / (n*(1-probs)))^(EVI[k.SCR]) * X.sort[(k.SCR+1)]
  ES.k.SCR    <- Q.k.SCR / (1 - min(0.9, EVI[k.SCR]))
  
  res.stoch <- matrix(c(EVI[k.MAX],   Q.k.MAX,  ES.k.MAX, k.MAX, 
                        EVI[k.SCR],   Q.k.SCR,  ES.k.SCR, k.SCR), 
                ncol=2, nrow=18, byrow=FALSE, 
                dimnames = list(c("EVI estimate", "Q 90", "Q 95", "Q 97.5", "Q 99", "Q 99.5", "Q 99.9", "Q 99.95", "Q 99.99", 
                                                   "ES 90", "ES 95", "ES 97.5", "ES 99", "ES 99.5", "ES 99.9", "ES 99.95", "ES 99.99", "k"), 
                                c("MAX", "SCR")))
  res.fix   <- c(EVI[k.fix], k.fix)
  return( list(res.stoch = res.stoch, res.fix = res.fix) )
}


