# ===================================================
# ===================================================
# Computing quantiles and tail indices of GARCH models in (M6)-(M7)
# ===================================================
# ===================================================


# ===================================================
# Computing true quantiles of GARCH(1,1)-models in (M6)-(M7)
# ===================================================


Quantile.true <- function(spec, alpha, n, reps){    # [Hil15, p.17 (Footnote 8)]
  no.alpha <- length(alpha)
  q.hat    <- ES.hat       <- matrix(NA, nrow=reps, ncol=no.alpha)
  for(i in 1 : reps){
    X <- as.numeric(garchSim(spec, n))
    X.partial <- sort.int(X, partial = floor(alpha[no.alpha] * n))
    X.partial <- sort.int(X.partial[1 : floor(alpha[no.alpha] * n)], method = "quick")
    for(a in 1:no.alpha){
      q.hat[i, a]     <- X.partial[floor(alpha[a] * n)] # [Hil15, p.6]
      ES.hat[i, a] <- 1/(1-alpha[a]) * mean( X * 1*(X >= q.hat[i, a]) )
    }
  }
  return( matrix(c(colMeans(q.hat), colMeans(ES.hat)), nrow=2, ncol=no.alpha, byrow=TRUE) )
}

# (M7)
spec.1 = garchSpec(model = list(omega=10e-06, alpha = 0.4, beta = 0.27), cond.dist = "norm")
spec.2 = garchSpec(model = list(omega=10e-06, alpha = 0.4, beta = 0.43), cond.dist = "norm")
spec.3 = garchSpec(model = list(omega=10e-06, alpha = 0.4, beta = 0.53), cond.dist = "norm")
spec.4 = garchSpec(model = list(omega=10e-06, alpha = 0.4, beta = 0.599), cond.dist = "norm")

# (M6)
spec.5 = garchSpec(model = list(omega=10e-06, alpha = 0.9, beta = 0.0), cond.dist = "norm")
spec.6 = garchSpec(model = list(omega=10e-06, alpha = 0.8, beta = 0.0), cond.dist = "norm")
spec.7 = garchSpec(model = list(omega=10e-06, alpha = 0.7, beta = 0.0), cond.dist = "norm")
spec.8 = garchSpec(model = list(omega=10e-06, alpha = 0.6, beta = 0.0), cond.dist = "norm")



library(snowfall)     # Parallel computing

sfInit(parallel = TRUE, cpus = 8, type = "SOCK")
sfLibrary(fGarch)
sfExport("Quantile.true")

wrapper <- function(spec){
  Quantile.true(spec, alpha = c(0.90, 0.95, 0.975, 0.99, 0.995, 0.999, 0.9995, 0.9999), n=100000, reps=100000)
}

start <- Sys.time() # takes 13 hours
Quantile.GARCH  <- sfLapply(c(spec.1, spec.2, spec.3, spec.4, spec.5, spec.6, spec.7, spec.8), wrapper)
print(Sys.time() - start)





# ===================================================
# Computing true quantiles of GARCH(1,1)-models in (M6)-(M7)
# ===================================================


f.GARCH.N01 <- function(alpha, alpha_1, beta_1){   # Calculate tail index of GARCH(1,1) with N(0,1)-innovations
  dens <- function(x) x^(alpha/2) * dchisq( (x - beta_1) / alpha_1, df = 1) / alpha_1  # dchisq(..) / .. is density of (alpha_1 * N(0,1)^2 + beta_1)
  EX <- integrate(dens, beta_1, Inf)$value
  return(EX - 1)
}



# (M6)
ti <- uniroot(f.GARCH.N01, interval = c(0.1, 10), tol = 1e-10, alpha_1 = 0.9, beta_1 = 0.0)
ti$root   # = 2.304337
ti <- uniroot(f.GARCH.N01, interval = c(0.1, 10), tol = 1e-10, alpha_1 = 0.8, beta_1 = 0.0)
ti$root   # = 2.684231
ti <- uniroot(f.GARCH.N01, interval = c(0.1, 10), tol = 1e-10, alpha_1 = 0.7, beta_1 = 0.0)
ti$root   # = 3.172043
ti <- uniroot(f.GARCH.N01, interval = c(0.1, 10), tol = 1e-10, alpha_1 = 0.6, beta_1 = 0.0)
ti$root   # = 3.821705 



# (M7)
ti <- uniroot(f.GARCH.N01, interval = c(0.1, 10), tol = 1e-10, alpha_1 = 0.4, beta_1 = 0.27)
ti$root   # = 4.992628
ti <- uniroot(f.GARCH.N01, interval = c(0.1, 10), tol = 1e-10, alpha_1 = 0.4, beta_1 = 0.43)
ti$root   # = 3.956559
ti <- uniroot(f.GARCH.N01, interval = c(0.1, 10), tol = 1e-10, alpha_1 = 0.4, beta_1 = 0.53)
ti$root   # = 2.975246
ti <- uniroot(f.GARCH.N01, interval = c(0.1, 10), tol = 1e-10, alpha_1 = 0.4, beta_1 = 0.599)
ti$root   # = 2.02789 



